package com.csc.library.database;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;

import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.UProfile;

public class ConnectDB {

	private HashMap transactionOn = new HashMap();
	
	private int hashCode = 0;
	
	public ConnectDB() {

	}
	
	public Connection getConnection(String dataBaseName) {
		try {
			ConnectionPools cp = (ConnectionPools) StaticProperties.connectionMap.get(dataBaseName.toLowerCase());
			Connection con = cp.getConnection(); 
			if (con!=null &&hashCode!=con.hashCode()) {
				hashCode =  con.hashCode();
			}
			return con;
		} catch (Exception ex) {
			MyLog.error(this, ex);
			return null;
		}
	}
	
	public Connection getConnection(String dataBaseName, Object userKey) throws SQLException{
		Connection con = null;
		String processid = ((UProfile)userKey).getProcessid();
		if(processid.length()>0) {
			CscConnection conprocess=null;
			if(ConnectionPools.connectionProcess.containsKey(processid)){
				conprocess=(CscConnection)ConnectionPools.connectionProcess.get(processid);
				if (conprocess.isClosed()) {
					ConnectionPools cp = (ConnectionPools) StaticProperties.connectionMap.get(dataBaseName.toLowerCase());
					conprocess = cp.createConnection();
					conprocess.setProcessid(processid);
					ConnectionPools.connectionProcess.put(((UProfile)userKey).getProcessid(), conprocess);
				}
			}else{
				ConnectionPools cp = (ConnectionPools) StaticProperties.connectionMap.get(dataBaseName.toLowerCase());
				conprocess = cp.createConnection();
				conprocess.setProcessid(processid);
				conprocess.setTransactionIsolation(Connection.TRANSACTION_READ_UNCOMMITTED);
				conprocess.setAutoCommit(true);
//				conprocess.setAutoCommit(false);
				ConnectionPools.connectionProcess.put(((UProfile)userKey).getProcessid(), conprocess);
			}
			return conprocess;
		}
		if (userKey != null) {
			String key = String.valueOf(((UProfile)userKey).getObjectID()).trim();
			if (transactionOn.containsKey(key)) {
				CscTransaction ts = (CscTransaction) transactionOn.get(key);
				if (!ts.isRollback()) {
					con = ts.getConnection(dataBaseName.toLowerCase(), userKey);
					return con;
				} else {
					MyLog.debug(this, dataBaseName+" :Transaction have rollbacked!" + userKey);
					SQLException x = new SQLException("Transaction have rollbacked!", "PIAK");
					transactionOn.remove(key);
					throw  x; 
				}
			} else {
				con = this.getConnection(dataBaseName.toLowerCase());
			}
		}
		if(con!=null){ //Ǥҧ  approve ˹ inbox
			con.setTransactionIsolation(Connection.TRANSACTION_READ_UNCOMMITTED);
			con.setAutoCommit(true);
//			con.setAutoCommit(false);
		}
		return con;
	}
	
	/**
	 * 
	 * @throws SQLException
	 * ͧ Connection  ˹ Transaction Tracking ӧҹ
	 */
	public void beginTransaction(Object userKey,  Object beginTransactionKey) throws SQLException {
//		synchronized(this.transactionOn){ // synchronized ͡ ͧҡԴ connection synchronization failed
			CscTransaction ts=null;
			String key = String.valueOf(((UProfile)userKey).getObjectID()).trim();
			if (!this.transactionOn.containsKey(key)) {
				ts=new CscTransaction();
				ts.userKey=key;
				if (userKey instanceof UProfile ) {
					((UProfile) userKey).clearSystemStatus(" ");
				}
				MyLog.debug(this, "("+ts.userKey + ") System Begin Transaction!");
				ts.beginTransactionKey = beginTransactionKey.hashCode();
				this.transactionOn.put(ts.userKey, ts);
			}
//		}
	}

	/**
	 * 
	 * @throws SQLException
	 * Commit data  ׹ Connection
	 */
	public void endTransaction(Object userKey,  Object endTransactionKey) throws SQLException {
//		synchronized(this.transactionOn){// synchronized ͡ ͧҡԴ connection synchronization failed
			CscTransaction ts;
			String key = String.valueOf(((UProfile)userKey).getObjectID()).trim();
			if (this.transactionOn.containsKey(key)) {
				ts = (CscTransaction) this.transactionOn.get(key);
				if (ts.beginTransactionKey==endTransactionKey.hashCode()) {
					if (!ts.isRollback()) {
						MyLog.debug(this, "("+key + ") System End Transaction!");
						ts.commit();
					}
					this.transactionOn.remove(key);
	//			comment by pas ͧҡ 1 process 1 connection ͧ stamp actiontime	
	//				} else {
	//				ts.setActionTime();
				}
//			  }
		 }
	}

	/**
	 * 
	 * @return Connection
	 * @throws SQLException
	 * Commit data  ׹ Connection
	 */
	public void rollback(Object userKey,  Object endTransactionKey) throws SQLException {
//		synchronized(this.transactionOn){// synchronized ͡ ͧҡԴ connection synchronization failed
			CscTransaction ts;
			String key = String.valueOf(((UProfile)userKey).getObjectID()).trim();
			if (this.transactionOn.containsKey(key)) {
				ts = (CscTransaction) this.transactionOn.get(key);
				ts.rollback();
				MyLog.error(this, "("+key + ") System Rollback Transaction!");
				this.transactionOn.remove(key);
			}
//		}
	}
}
