
package com.csc.library.security;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.Provider;
import java.security.Security;
import java.util.HashMap;

import javax.crypto.SecretKey;

import org.jdom.Element;

import com.csc.library.database.StaticProperties;
import com.csc.library.session.InitialEnvironment;
import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.ReadXml;



public class LoadTemplate {
	public String user="",password="";
	public LoadTemplate() {
		Provider sunJce =new com.sun.crypto.provider.SunJCE();
		Security.addProvider(sunJce);
		//java.Security.addProvider(sunJce);
	}
	public void process(){
		this.processKey();
		this.processData();
		if(StaticProperties.dbuser.equals(""))
			this.setSystemData();
			
	}
	private void processKey(){
		HashMap data=this.loadTemplateKey();
		if(!data.isEmpty()){
			this.setOriginalData();
			this.writeMasterKey(data);
			this.writeBKey();
			this.resetDataForNewKey();
			MyLog.info(this,"security key changed !!");	
		}
	}
	

	private void processData(){
		String jcePath=new InitialEnvironment("GLOBAL").getValue("jce-dir");
		Element em=this.loadTemplate(jcePath+"template/DB_SECURITY.xml");
		String user=em.getChild("USER").getText().trim();			
		if(!user.equals("")){
			this.writeMasterData();
			this.writeBTemplate();
			this.setSystemData();
			MyLog.info(this,"security data changed !!");
		}
	}
	
	private void setSystemData(){
		try {
			String tempPath=new InitialEnvironment("GLOBAL").getValue("temp-dir");
			HashMap data=this.loadMasterKey();
			SecretKey key=(SecretKey)data.get("key");
			EncryptText encrypter=new EncryptText(key); 
			encrypter.decrypt(new FileInputStream(tempPath+"data/data.txt"),
			new FileOutputStream(tempPath+"data/tmp.xml"));
			Element em=this.loadTemplate(tempPath+"data/tmp.xml");
			StaticProperties.dbuser=em.getChild("USER").getText().trim();
			StaticProperties.dbpassword=em.getChild("PASSWORD").getText().trim();
		    File f=new File(tempPath+"data/tmp.xml");
		    f.delete();		
		} catch (Exception e) {
			MyLog.error(this, e);
		}		
	}
	
	private void setOriginalData(){
		try {
			String tempPath=new InitialEnvironment("GLOBAL").getValue("temp-dir");
			HashMap data=this.loadMasterKey();
			if(data!=null){
			SecretKey key=(SecretKey)data.get("key");
			EncryptText encrypter=new EncryptText(key); 
	        encrypter.decrypt(new FileInputStream(tempPath+"data/data.txt"),
		    new FileOutputStream(tempPath+"data/tmp.xml"));								
			}
		}catch (Exception e) {
			MyLog.error(this, e);
		}		
	}
	
	private void resetDataForNewKey(){
		try {
			String tempPath=new InitialEnvironment("GLOBAL").getValue("temp-dir");
			HashMap data=this.loadMasterKey();
			SecretKey key=(SecretKey)data.get("key");
			EncryptText encrypter=new EncryptText(key); 			
		    encrypter.encrypt(new FileInputStream(tempPath+"data/tmp.xml"),
			new FileOutputStream(tempPath+"data/data.txt"));			
		    File f=new File(tempPath+"data/tmp.xml");
		    f.delete();
		}catch (Exception e) {
			MyLog.error(this, e);
		}				
	}
	private Element loadTemplate(String path){
		ReadXml w=new ReadXml();
		w.setDataManager(path);
		return w.getRootElement();
	}
	private  HashMap loadTemplateKey(){
		try {
			String jcePath=new InitialEnvironment("GLOBAL").getValue("jce-dir");
			HashMap data=new HashMap();
			FileInputStream fu=new FileInputStream(jcePath+"template/key.ser");
			ObjectInputStream ou=new ObjectInputStream(fu);
			data=(HashMap)ou.readObject();
			return data;
		} catch (Exception e) {
			MyLog.error(this, e);
			return null;
		}		
	}
	private  HashMap loadMasterKey(){
		try {
			String configPath=new InitialEnvironment("GLOBAL").getValue("config-dir");
			HashMap data=new HashMap();
			FileInputStream fu=new FileInputStream(configPath+"data/data.txt");
			ObjectInputStream ou=new ObjectInputStream(fu);
			data=(HashMap)ou.readObject();
			return data;
		} catch (Exception e) {
			MyLog.error(this, e);
			return null;
		}				
	}
	private void writeBTemplate(){
		String jcePath=new InitialEnvironment("GLOBAL").getValue("jce-dir");
		String bTemplate="<DB_SECURITY>\n";
		bTemplate+="<USER></USER>\n";
		bTemplate+="<PASSWORD></PASSWORD>\n";
		bTemplate+="</DB_SECURITY>";
		try {
			FileOutputStream fu=new FileOutputStream(jcePath+"template/DB_SECURITY.xml");
			fu.write(bTemplate.getBytes());
		} catch (Exception e) {
			MyLog.error(this, e);
		}			
	}
	
	private void writeMasterData(){
		try {
			String tmpPath=new InitialEnvironment("GLOBAL").getValue("temp-dir");
			String jcePath=new InitialEnvironment("GLOBAL").getValue("jce-dir");
			HashMap data=this.loadMasterKey();
			SecretKey key=(SecretKey)data.get("key");
			EncryptText encrypter=new EncryptText(key); 
		    encrypter.encrypt(new FileInputStream(jcePath+"template/DB_SECURITY.xml"),
	        new FileOutputStream(tmpPath+"data/data.txt"));			
		} catch (Exception e) {
			MyLog.error(this, e);
		}			
	}	
	
	private void writeBKey(){
		try {
			String jcePath=new InitialEnvironment("GLOBAL").getValue("jce-dir");
			HashMap data=new HashMap();
			FileOutputStream fu=new FileOutputStream(jcePath+"template/key.ser");
			ObjectOutputStream ou=new ObjectOutputStream(fu);
			ou.writeObject(data);
		} catch (Exception e) {
			MyLog.error(this, e);
		}		
	}
	private void writeMasterKey(HashMap data){
		try {
			String configPath=new InitialEnvironment("GLOBAL").getValue("config-dir");
			FileOutputStream fu=new FileOutputStream(configPath+"data/data.txt");
			ObjectOutputStream ou=new ObjectOutputStream(fu);
			ou.writeObject(data);
		} catch (Exception e) {
			MyLog.error(this, e);
		}		
	}	
	public static void main(String[] args) {
		LoadTemplate load=new LoadTemplate();
		load.process();
		System.out.println("USER : "+load.user);
		System.out.println("PASSWORD : "+load.password);
	}
}
