package com.csc.library.security;

import java.security.MessageDigest;

import sun.security.provider.SHA;

import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;
import com.sun.org.apache.xml.internal.serializer.ToStream;

public class SHAEncoder {

	private boolean verbose = false;

	private MessageDigest sha = null;
	
	private static String SHA1="SHA-1";
	
	private static String SALT="csc-sha"; 
	
	//public static String LABEL="{CSC-SHA1}";

	/**
	* public constructor
	*/
	public  SHAEncoder() {
		verbose = false;
		try {
			sha = MessageDigest.getInstance(SHAEncoder.SHA1);
		} catch (java.security.NoSuchAlgorithmException e) {
			System.out.println("Construction failed: " + e);
		}
	}
	
	/**
	 * Fast convert a byte array to a hex string with possible leading zero.
	 * @param b is byte array to convert to hexString
	 * @return String representation of byte array
	 */
	private static String toHexString(byte[] b) {
		final char[] hexChar = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
		StringBuffer sb = new StringBuffer(b.length * 2);
		for (int i = 0; i < b.length; i++) {
			// look up high nibble char
			sb.append(hexChar[(b[i] & 0xf0) >>> 4]);
			// look up low nibble char
			sb.append(hexChar[b[i] & 0x0f]);
		}
		return sb.toString();
	}

	/**
	* Create Digest for each entity values passed in
	*
	* @param salt
	* byte array to set the base for the encryption
	* @param entity
	* string to be encrypted
	* @return string representing the salted hash output of the encryption
	* operation
	*/
	public String createDigest(byte[] salt, String entity) {

		// Update digest object with byte array of the source clear text
		// string and the salt
		sha.reset();
		sha.update(entity.getBytes());
		sha.update(salt);

		// Complete hash computation, this results in binary data
		byte[] pwhash = sha.digest();

		if (verbose) {
			System.out.println("Digest password (hex): "+ toHexString(pwhash) );
			System.out.println("nSalt: "+ toHexString(salt) );
			System.out.println("Putting it all together: ");
			System.out.println("Binary digest password + binary salt: "	+ pwhash + salt);
		}

		return new String(Base64.encode(concat(pwhash, salt))).trim();
	}
	
	/**
	* Create Digest for each entity values passed in
	*
	* byte array to set the base for the encryption
	* @param entity
	* string to be encrypted
	* @return string representing the salted hash output of the encryption
	* operation
	*/	
	public String createDigest( String entity) {

		// Update digest object with byte array of the source clear text
		// string and the salt
		sha.reset();
		sha.update(entity.getBytes());
		sha.update(SHAEncoder.SALT.getBytes());

		// Complete hash computation, this results in binary data
		byte[] pwhash = sha.digest();

		if (verbose) {
			System.out.println("Digest password (hex): "+ toHexString(pwhash) );
			System.out.println("nSalt: "+ SHAEncoder.SALT );
			System.out.println("Putting it all together: ");
			System.out.println("Binary digest password + binary salt: "	+ pwhash + SHAEncoder.SALT);
		}
		return new String(Base64.encode(concat(pwhash, SHAEncoder.SALT.getBytes()))).trim();
	}
	
	public String createDigestNoSalt( String entity) {
		sha.reset();
		sha.update(entity.getBytes());
		byte[] pwhash = sha.digest();
		String result=toHexString(pwhash);
		if (verbose) {
			System.out.println("Digest password (hex): "+ toHexString(pwhash) );
			System.out.println("Putting it all together: ");
		}
		return result.trim();
	}

	/**
	* Check Digest against entity
	*
	* @param digest
	* is digest to be checked against
	* @param entity
	* entity (string) to be checked
	* @return TRUE if there is a match, FALSE otherwise
	*/
	public boolean checkDigest(String digest, String entity) {
		boolean valid = true;
		// ignore the {SSHA} hash ID
//		if(digest.startsWith(SHAEncoder.LABEL))
//			digest = digest.substring(10);

		// extract the SHA hashed data into hs[0]
		// extract salt into hs[1]
		byte[][] hs = split(Base64.decode(digest), 20);
		byte[] hash = hs[0];
		byte[] salt = hs[1];

		// Update digest object with byte array of clear text string and salt
		sha.reset();
		sha.update(entity.getBytes());
		sha.update(salt);

		// Complete hash computation, this is now binary data
		byte[] pwhash = sha.digest();

		if (verbose) {
			System.out.println("Salted Hash extracted (hex): " + toHexString(hash) );
			System.out.println("nSalt extracted (hex): " + toHexString(salt));
			System.out.println("Hash length is: " + hash.length+ " Salt length is: " + salt.length);
			System.out.println("Salted Hash presented (hex): " + toHexString(pwhash));
		}

		if (!MessageDigest.isEqual(hash, pwhash)) {
			valid = false;
			System.out.println("Hashes DON'T match: " + entity);
		}

		if (MessageDigest.isEqual(hash, pwhash)) {
			valid = true;
			System.out.println("Hashes match: " + entity);
		}

		return valid;
	}
	
	/**
	* set the verbose flag
	*/
	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	/**
	* Combine two byte arrays
	*
	* @param l
	*      first byte array
	* @param r
	*      second byte array
	* @return byte[] combined byte array
	*/
	private static byte[] concat(byte[] l, byte[] r) {
		byte[] b = new byte[l.length + r.length];
		System.arraycopy(l, 0, b, 0, l.length);
		System.arraycopy(r, 0, b, l.length, r.length);
		return b;
	}

	/**
	* split a byte array in two
	*
	* @param src
	*      byte array to be split
	* @param n
	*      element at which to split the byte array
	* @return byte[][] two byte arrays that have been split
	*/
	private static byte[][] split(byte[] src, int n) {
		byte[] l, r;
		if (src == null || src.length <= n) {
			l = src;
			r = new byte[0];
		} else {
			l = new byte[n];
			r = new byte[src.length - n];
			System.arraycopy(src, 0, l, 0, n);
			System.arraycopy(src, n, r, 0, r.length);
		}
		byte[][] lr = { l, r };
		return lr;
	}

	private static String hexits = "0123456789abcdef";



	/**
	* Convert a String hex notation to a byte array
	*
	* @param s string to convert
	* @return byte array
	*/
	private static byte[] fromHex(String s) {
		s = s.toLowerCase();
		byte[] b = new byte[(s.length() + 1) / 2];
		int j = 0;
		int h;
		int nibble = -1;

		for (int i = 0; i < s.length(); ++i) {
			h = hexits.indexOf(s.charAt(i));
			if (h >= 0) {
				if (nibble < 0) {
					nibble = h;
				} else {
					b[j++] = (byte) ((nibble << 4) + h);
					nibble = -1;
				}
			}
		}

		if (nibble >= 0) {
			b[j++] = (byte) (nibble << 4);
		}

		if (j < b.length) {
			byte[] b2 = new byte[j];
			System.arraycopy(b, 0, b2, 0, j);
			b = b2;
		}

		return b;
	}
	
	public boolean validSalt(String data){
	   try{
		String mySalt=new String(Base64.decode(data));
		if(mySalt.endsWith(SHAEncoder.SALT)){
			return true;
		}else{
			return false;
		}
	   }catch(Exception ex){
		   return false;
	   }
	}

	public static void main(String[] args) {
		SHAEncoder sh = new SHAEncoder();
		sh.setVerbose(true);
		// generate digest from data passed in
		System.out.println("-------"+sh.createDigest("systemadmin"));
//		System.out.println("-------------------------"+sh.createDigestNoSalt("csc"));
//		System.out.println(sh.createDigest(sh.createDigestNoSalt("csc")));
//		System.out.print("-------------------------"+new String(Base64.encode("slksjskjls".getBytes())));
		
		//System.out.println(new SHAEncoder().createDigest("bbbbbbbb1"));		
		//validate data against digest
//		System.out.println(sh.checkDigest(new SHAEncoder().createDigest("000000"),"v1Z6CNOG0zjquMBRz2uK+lWn+zBjc2Mtc2hh"));
//		System.out.println(sh.checkDigest("ciOC4iz9j9vQZj1XrGAPA9+bi/pjc2Mtc2hh", sh.createDigestNoSalt("000000")));
//		System.out.println(sh.validSalt("v1Z6CNOG0zjquMBRz2uK+lWn+zBjc2Mtc2hh"));
		
	}
}