package com.csc.library.session;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

import com.csc.library.database.ChildManager;
import com.csc.library.database.ConfigMetadata;
import com.csc.library.database.ConnectManager;
import com.csc.library.database.FieldGroup;
import com.csc.library.database.MyRecordGroup;
import com.csc.library.database.Relation;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscException;
import com.csc.library.utilities.UProfile;


public interface DbTable extends ChildManager {
	public void refresh() throws RemoteException;
	public void next() throws RemoteException;
	public void previous() throws RemoteException;
	public void first() throws RemoteException;
	public DbRecord getRecordManager()throws RemoteException;
	public void last() throws RemoteException;
	public int recCount() throws RemoteException;
	public int recNo() throws RemoteException;
	public boolean isFirst() throws RemoteException;
	public boolean isLast() throws RemoteException;
	public boolean eof() throws RemoteException;
	public void clear() throws RemoteException;
	public int save() throws RemoteException;
	public int delete() throws RemoteException;
	public CscCalendar getCscCalendar(String fieldName) throws RemoteException;
	public String getLastUpdate() throws RemoteException;
	public void restore() throws RemoteException;
	public void print() throws RemoteException;
	public void printAll() throws RemoteException;
	public String getString(String fieldName) throws RemoteException;
	public boolean getBoolean(String fieldName) throws RemoteException;
	public byte getByte(String fieldName) throws RemoteException;
	public char[] getChars(String fieldName) throws RemoteException;
	public char getChar(String fieldName) throws RemoteException;
	public short getShort(String fieldName) throws RemoteException;
	public int getInt(String fieldName) throws RemoteException;
	public long getLong(String fieldName) throws RemoteException;
	public float getFloat(String fieldName) throws RemoteException;
	public double getDouble(String fieldName) throws RemoteException;
	public Timestamp getTimestamp(String fieldName) throws RemoteException;
	public BigDecimal getBigcimal(String fieldName) throws RemoteException;
	public Time getTime(String fieldName) throws RemoteException;
	public java.sql.Date getDate(String fieldName) throws RemoteException;
	public boolean set(String fieldName, String data) throws RemoteException;
	public boolean set(String fieldName, boolean data) throws RemoteException;
	public boolean set(String fieldName, byte data) throws RemoteException;
	public boolean set(String fieldName, char data[]) throws RemoteException;
	public boolean set(String fieldName, char data) throws RemoteException;
	public boolean set(String fieldName, short data) throws RemoteException;
	public boolean set(String fieldName, int data) throws RemoteException;
	public boolean set(String fieldName, long data) throws RemoteException;
	public boolean set(String fieldName, float data) throws RemoteException;
	public boolean set(String fieldName, double data) throws RemoteException;
	public boolean set(String fieldName, Timestamp data) throws RemoteException;
	public boolean set(String fieldName, java.sql.Date data) throws RemoteException;
	public boolean set(String fieldName, BigDecimal data) throws RemoteException;
	public boolean set(String fieldName, Time data) throws RemoteException;
	public String getError() throws RemoteException;
	public void setCondition() throws RemoteException;
	public void setKey(String data) throws RemoteException;
	public void setKey(String data[]) throws RemoteException;
	public boolean setColumn(String columnList) throws RemoteException;
	public boolean setColumn(Vector columnList) throws RemoteException;
	public String getListKey() throws RemoteException;
	public void setUsedFor(char usedFor) throws RemoteException;
	public char getUsedFor() throws RemoteException;
	public String getTableName() throws RemoteException;
	public void setTableName(String name) throws RemoteException;
	public void setActualName(String name) throws RemoteException;
	public int setCondition(String condition[]) throws RemoteException;
	public void search() throws RemoteException;
	public void addRecord() throws RemoteException;
	public DbRecord addRecord(String key) throws CscException, RemoteException;
	public void deleteRecord() throws RemoteException;
	public void deleteRecord(String key) throws RemoteException;
	public Hashtable getChildTable() throws RemoteException;
	public void setChildTable(Hashtable hashChild) throws RemoteException;
	public Hashtable getRelationInfo() throws RemoteException;
	public void setRelationInfo(Hashtable hashRelation) throws RemoteException;
	public void putRelation(Relation oRela) throws RemoteException;
	public void removeRelation(String tableName) throws RemoteException;
	public Relation getRelation(String tableName) throws RemoteException;
	public int putChild(ChildManager oChild) throws RemoteException;
	public int putChild(String childName, char type, char usedFor) throws RemoteException;
	public void removeChild(String childName) throws RemoteException;
	public void refreshChild() throws RemoteException;
	public void refreshChild(String childName) throws RemoteException;
	public ChildManager getChild(String childName) throws RemoteException;
	public DbRecord getRecordManager(String childName) throws RemoteException;
	public DbRecord getCurrentRecordManager() throws RemoteException;
	public DbRecord getCurrentRecordManager(int recNo) throws RemoteException;
	public DbRecord getCurrentRecordManager(String key)throws Exception;
	public DbTable getTableManager(String childName) throws RemoteException;
	public String listChild() throws RemoteException;
	public void setAutoRefresh(byte status) throws RemoteException;
	public byte getAutoRefresh() throws RemoteException;
	public Hashtable getEffectTable() throws RemoteException;
	public void setEffectTable(Hashtable hashEffectTable) throws RemoteException;
	public int setMinorKey(String keyName) throws RemoteException;
	public String getMinorKey() throws RemoteException;
	public void setKeyOff() throws RemoteException;
	public void setKeyOn() throws RemoteException;
	public boolean getKeyStatus() throws RemoteException;
	public void setPropDB(String comp, String user, String pwd) throws RemoteException;
	public String getFieldList() throws RemoteException;

	public void setOrderBy(String o) throws RemoteException;
//	public void setFilter(String s) throws RemoteException;
	public DbRecord getNewRecord() throws RemoteException;
	public void putNewRecord(DbRecord rec) throws RemoteException;
	public void putNewRecord(DbRecord rec,int no) throws RemoteException;	
	public void setLastUpdate() throws RemoteException;

	public int fieldGroupCount() throws RemoteException;
	public Vector getAllFieldGroupName() throws RemoteException;
	public void putFieldGroup(FieldGroup gf) throws RemoteException;
	public FieldGroup getFieldGroup(String groupCode) throws RemoteException;
	public String getLinkFieldChild(String childName) throws RemoteException;
	public String getLinkFieldParent(String childName) throws RemoteException;
	public boolean setColumn(FieldGroup fg) throws RemoteException;
	public boolean addChildColumn(String childName) throws RemoteException;
	public void setFieldGroupColumn(String groupCode) throws RemoteException;
	public boolean addColumn(String columnList) throws RemoteException;
	public void setRefreshModel(byte model) throws RemoteException;
	public void setJSPString(String fieldName, String data) throws RemoteException;
	public String getJSPString(String fieldName) throws RemoteException;
	public void setConnectControl(ConnectManager con) throws RemoteException;
	//public connectManager getConnectControl()throws RemoteException;
	public void setAllRequiredFound(boolean require) throws RemoteException;
	public void setRequiredFound(boolean require) throws RemoteException;
	public boolean getRequiredFound() throws RemoteException;
	public void setAllUProfile(UProfile uPro) throws RemoteException;
	public void initMyTable(String className, String condition, String filter, String order) throws RemoteException;
	public void referLangOff() throws RemoteException;
	public void referLangOn() throws RemoteException;
	
	public void setParentDb(DbRecord db)throws RemoteException;
	public DbRecord getParentDb()throws RemoteException;
	public void setRelationLink(Vector parent,Vector child)throws RemoteException;
	public void clearRecord()throws RemoteException;
	
	public void setParam(HashMap param)throws RemoteException;
	public HashMap getParam()throws RemoteException;
	public void setParam(String key,String value)throws RemoteException;
	public String getParam(String key)throws RemoteException;
	
    public boolean realDelete() throws RemoteException;
	public boolean realDelete(String filter) throws RemoteException;
    public void setKeyGroup(String keyGroup)throws RemoteException;
    public String getMessage()throws RemoteException;
    public void setMessage(String message)throws RemoteException;
//    public void setTranSaction(javax.transaction.UserTransaction transaction)throws RemoteException;
    public DbRecord getRecordForSave() throws RemoteException;
	public void setGroupfield(String gField)throws RemoteException;    
	public HashMap getGroupField()throws RemoteException;			
	public void setVMetaData(ConfigMetadata vm) throws RemoteException;
	public  void setLoasComponent(boolean load);
	public void formatOn(); 
	public void formatOff();
	public void setFieldFormat(String type);
	public String getAllData(); 
	public void setVerifyOff();
	public String getStringHTML(String fieldName);
	public void setBlankRecord(int amt);
	public int getBlankRecord();
	public String getDataFormat(String fieldName);
	public void goRec(int no,boolean isRefreshChild);
	public String getDbType();
	//create by pas  trace object
	public String getObjectTrace();
	/* create for transfer workflow*/
    public void setWFTransferOn(String wf_ref_doc);
    public void setWFTransferOff();
	public boolean getWFTransfer();	
	public String getWFRefDoc();    
	public MyRecordGroup getMyRecords();
	public int getCurrentRecordNumber();
}