package com.csc.library.utilities;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.StringTokenizer;

public class CscCalendar implements Serializable, Cloneable {
	private static final long	serialVersionUID	= 1L;	
	private GregorianCalendar	calendar			= new GregorianCalendar();
	//private Calendar calendar = Calendar.getInstance(); //.getInstance();
	private DateFormat			dateFormat			= DateFormat.getInstance();
	private DecimalFormat		dicimalFormat;
	private int					myEra				= 1;
	private String[]			thaiLongDay			= { "ҷԵ", "ѹ", "ѧ", "ظ", "ʺ", "ء", "" };
	private String[]			thaiLongMonth		= { "Ҥ", "Ҿѹ", "չҤ", "¹", "Ҥ", "Զع¹", "áҤ", "ԧҤ", "ѹ¹", "Ҥ", "Ȩԡ¹", "ѹҤ" };
	private String[]			thaiShortDay		= { "", "", "", "", "", "", "" };
	private String[]			thaiShortMonth		= { "..", "..", "..", "..", "..", "..", "..", "..", "..", "..", "..", ".." };
	private String[]			engShortDay			= { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
	private String[]			engLongDay			= { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" };
	private String[]			engShortMonth		= { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	private String[]			engLongMonth		= { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
	private String				d30					= "04 06 09 11";
	private String				d31					= "01 03 05 07 08 10 12";
	private String				sign				= "-";
	private int[]				monthCount			= { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

	private long				dayOfTimeStamp		= 86400000;
	private long				hourOfTimeStamp		= dayOfTimeStamp / 24;
	private long				minuteOfTimeStamp	= hourOfTimeStamp / 60;
	private long				secondOfTimeStamp	= minuteOfTimeStamp / 60;
	private boolean				todogol				= false;
	public CscCalendar(String s) {
		this.setCalendar(s);
	}
	public CscCalendar(int year, int month, int date) {
		this.calendar.set(Calendar.YEAR, year);
		this.calendar.set(Calendar.MONTH, month -1);
		this.calendar.set(Calendar.DATE, date);
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar(int year, int month, int date, int hour, int minute) {
		this.calendar.set(Calendar.YEAR, year);
		this.calendar.set(Calendar.MONTH, month - 1);
		this.calendar.set(Calendar.DATE, date);
		this.calendar.set(Calendar.HOUR, hour);
		this.calendar.set(Calendar.MINUTE, minute);
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar(int year, int month, int date, int hour, int minute, int second) {
		this.calendar.set(Calendar.YEAR, year);
		this.calendar.set(Calendar.MONTH, month);
		this.calendar.set(Calendar.DATE, date);
		this.calendar.set(Calendar.HOUR, hour);
		this.calendar.set(Calendar.MINUTE, minute);
		this.calendar.set(Calendar.SECOND, second);
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar(java.sql.Timestamp timestamp) {
		this.calendar.setTime((Date) timestamp);
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar(double time) {
		this.calendar.setTime(new Timestamp(Double.doubleToRawLongBits(time)));
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar(long time) {
		this.calendar.setTime(new Timestamp(time));
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar(java.util.Date date) {
		this.calendar.setTime(date);
		this.dateFormat.setCalendar(this.calendar);
	}
	public CscCalendar() {
		this.dateFormat.setCalendar(this.calendar);
	}
	//---------------------------------------------------------------------------------------------------------------------------------------
	// SUB AND COUNT
	//----------------------------------------------------------------------------------------------------------------------------------------
	public int subForDay(CscCalendar calendar) {
		int y, d;
		y = this.calendar.get(Calendar.YEAR) > calendar.getYear() ? calendar.getYear() : this.calendar.get(Calendar.YEAR);
		d = this.countDay(y) - calendar.countDay(y);
		return d;
	}
	public float subForHour(CscCalendar calendar) {
		int d, h, m;
		float f;
		d = this.subForDay(calendar) * 24 * 60;
		m = d + this.getCSCMinute() - calendar.getCSCMinute();
		h = (int) m / 60;
		f = h + ((float) (m % 60) / 100);
		return f;
	}
	private int countDay(int y) {
		CscCalendar begcalendar = new CscCalendar(String.valueOf(y) + "-12-31 00:00:00.00");
		int c = 0;
		//System.out.println("begcalendar.getYear()"+begcalendar.getYear());
		//System.out.println("this.calendar.get(Calendar.YEAR)="+this.calendar.get(Calendar.YEAR));
		for (; begcalendar.getYear() < this.calendar.get(Calendar.YEAR); begcalendar.incYear(1)) {
			c += begcalendar.getDayOfYear();
		}
		c += this.calendar.get(Calendar.DAY_OF_YEAR);
		return c;
	}
	public boolean isYear366() {
		if ((this.getYear() % 4) == 0)
			return true;
		else
			return false;
	}
	protected int countDay(long start, long end) {
		long result = 0;
		if (end > start)
			result = (end - start) / 86400000;
		else
			result = (start - end) / 86400000;
		Long l = new Long(result);
		return Math.abs(l.intValue() + 1);
	}
	public int countDay(CscCalendar carlendar) {
		return this.countDay(this.getTime(), carlendar.getTime());
	}
	public int countDay(String day) {
		return this.countDay(new CscCalendar(day));
	}
	public float subForDecimal(CscCalendar calendar) {
		int d, h, m;
		float f;
		d = this.subForDay(calendar) * 24 * 60;
		m = d + this.getCSCMinute() - calendar.getCSCMinute();
		h = (int) m / 60;
		f = h + ((float) (m % 60) / 60);
		return f;
	}
	//-----------------------------------------------------------------------------------------------------------------------------------------
	// DEC  AND INC
	//---------------------------------------------------------------------------------------------------------------------------------------
	public CscCalendar incDate() {
		this.calendar.add(Calendar.DATE, 1);
		return this;
	}
	public CscCalendar decDate() {
		this.calendar.add(Calendar.DATE, -1);
		return this;
	}
	public CscCalendar incDate(int day) {
		this.calendar.add(Calendar.DATE, day);
		return this;
	}
	public CscCalendar incYear(int year) {  
		 if (year >0) {
			 this.calendar.add(Calendar.YEAR, year);
		}
		/* comment by pas ͧҡ calendar 繵ͧ令ӹǳͧ
		 * if (year != 0) {
			int max = this.getYear() + year;
			for (int i = this.getYear(); i < max; i++) {
				if (this.isLeapYear(i))
					this.calendar.add(Calendar.DATE, 366);
				else
					this.calendar.add(Calendar.DATE, 365);
			}
		}*/
		return this;
	}
	public CscCalendar decDate(int day) {
		this.calendar.add(Calendar.DATE, -day);
		return this;
	}
	public boolean isInRange(CscCalendar start, CscCalendar end) {
		if (this.afterEquals(start) && this.beforeEquals(end))
			return true;
		else
			return false;
	}
	public boolean isInRange(Date start, Date end) {
		return this.isInRange(new CscCalendar(start), new CscCalendar(end));
	}
	public boolean isInRange(String start, String end) {
		return this.isInRange(new CscCalendar(start), new CscCalendar(end));
	}
	public CscCalendar decYear(int year) {
		 if (year >0) {
			 this.calendar.add(Calendar.YEAR, (year*-1));
		}		 
		/*if (year != 0) {
			int max = this.getYear() + year;
			for (int i = this.getYear(); i < max; i++) {
				if (this.isLeapYear(i))
					this.calendar.add(Calendar.DATE, -366);
				else
					this.calendar.add(Calendar.DATE, -365);
			}
		}*/
		return this;
	}
	public CscCalendar incHour() {
		this.calendar.add(Calendar.HOUR, 1);
		return this;
	}
	public CscCalendar decHour() {
		this.calendar.add(Calendar.HOUR, -1);
		return this;
	}
	public CscCalendar incHour(int hour) {
		this.calendar.add(Calendar.HOUR, hour);
		return this;
	}
	public CscCalendar decHour(int hour) {
		this.calendar.add(Calendar.HOUR, -hour);
		return this;
	}
	public CscCalendar incMinute() {
		this.calendar.add(Calendar.MINUTE, 1);
		return this;
	}
	public CscCalendar decWeek() {
		for (int i = 0; i < 7; i++)
			this.incDate();
		return this;
	}
	public CscCalendar decWeek(int week) {
		if (week > 0) {
			for (int i = 0; i < week; i++)
				this.decWeek();
		} else {
			MyLog.warn(this, "Please Input Week More Than 0");
		}
		return this;
	}
	public CscCalendar incWeek() {
		for (int i = 0; i < 7; i++)
			this.incDate();
		return this;
	}
	public CscCalendar incWeek(int week) {
		if (week > 0) {
			for (int i = 0; i < week; i++)
				this.incWeek();
		} else {
			MyLog.warn(this, "Please Input Week More Than 0");
		}
		return this;
	}
	public CscCalendar incMonth(int m) {
		this.calendar.add(Calendar.MONTH, m);
		return this;
	}
	public CscCalendar decMonth() {
		this.calendar.add(Calendar.MONTH, -1);
		return this;
	}
	public CscCalendar decMonth(int m) {
		if (m > 0) {
			this.calendar.add(Calendar.MONTH, -1 * m);
		} else {
			this.calendar.add(Calendar.MONTH, m);
		}
		return this;
	}
	public CscCalendar decMinute() {
		this.calendar.add(Calendar.MINUTE, -1);
		return this;
	}
	public CscCalendar incMinute(int minute) {
		this.calendar.add(Calendar.MINUTE, minute);
		return this;
	}
	public CscCalendar decMinute(int minute) {
		this.calendar.add(Calendar.MINUTE, -minute);
		return this;
	}
	public boolean isLeapYear(int year) {
		return this.calendar.isLeapYear(year);
	}
	public boolean isLeapYear(CscCalendar calendar) {
		return this.calendar.isLeapYear(calendar.getYear());
	}
	public boolean isLeapYear() {
		return this.calendar.isLeapYear(this.calendar.get(Calendar.YEAR));
	}
	public void incTime(float hour) {
		//System.out.println("+++ hour="+hour+" +++ (int) hour="+(int) hour+" +++ (int) hour * 100="+(int) (hour * 100)+" +++ minius="+((int) (hour * 100) - ((int) hour)*100)   );
		this.calendar.add(Calendar.HOUR, (int) hour);
		this.calendar.add(Calendar.MINUTE, ((int) (hour * 100) - ((int) hour) * 100));
	}
	public CscCalendar decTime(float hour) {
		//System.out.println("+++ hour="+hour+" +++ (int) hour="+(int) hour+" +++ (int) hour * 100="+(int) (hour * 100)+" +++ minius="+((int) (hour * 100) - ((int) hour)*100)  );
		this.calendar.add(Calendar.HOUR, -(int) hour);
		this.calendar.add(Calendar.MINUTE, -((int) (hour * 100) - ((int) hour) * 100));
		return this;

	}
	public CscCalendar incDecimal(float hour) {
		this.calendar.add(Calendar.HOUR, (int) hour);
		this.calendar.add(Calendar.MINUTE, (int) (hour - (int) hour * 60));
		return this;
	}
	public CscCalendar decDecimal(float hour) {
		this.calendar.add(Calendar.HOUR, -(int) hour);
		this.calendar.add(Calendar.MINUTE, -(int) (hour - (int) hour * 60));
		return this;
	}
	//------------------------------------------------------------------------------------------------------------------------------------
	// BOOLEAN CHECK
	//-------------------------------------------------------------------------------------------------------------------------------------
	public boolean beforeDate(CscCalendar calendar) {
		Timestamp tmp = new Timestamp(this.getDay());
		Timestamp tmp2 = new Timestamp(calendar.getDay());
		return tmp.before(tmp2);
	}

    public boolean afterDateTime(CscCalendar calendar) {
        Timestamp tmp = new Timestamp(this.getTime());
        return tmp.after(calendar.getTimestamp());
    }
    
    public boolean afterDate(CscCalendar calendar) {
		Timestamp tmp = new Timestamp(this.getDay());
		Timestamp tmp2 = new Timestamp(calendar.getDay());
		return tmp.after(tmp2);
	}

    public boolean beforeDateTime(CscCalendar calendar) {
        Timestamp tmp = new Timestamp(this.getTime());
        return tmp.before(calendar.getTimestamp());
    }    
    
    public boolean equalsDate(CscCalendar calendar) {
		Timestamp tmp = new Timestamp(this.getDay());
		return tmp.equals(new Timestamp(this.getDay(calendar)));
	}

    public boolean equalsDateTime(CscCalendar calendar) {
        Timestamp tmp = new Timestamp(this.getTime());
        return tmp.equals(calendar.getTimestamp());
    }
    /************* Nu Ball Edit 28-10-48**********/	
	public boolean beforeEqualsDate(CscCalendar calendar) {
		return this.equalsDate(calendar) || this.beforeDate(calendar);
	}
    
	public boolean afterEqualsDate(CscCalendar calendar) {
		return this.equalsDate(calendar) || this.afterDate(calendar);
	}

    public boolean beforeEqualsDateTime(CscCalendar calendar) {
        return this.equalsDateTime(calendar) || this.beforeDateTime(calendar);
    }
    
    public boolean afterEqualsDateTime(CscCalendar calendar) {
        return this.equalsDateTime(calendar) || this.afterDateTime(calendar);
    }
    
    /*	public boolean beforeEqualsDate(CscCalendar calendar) {
		return this.equalsDate(calendar) || this.calendar.before(calendar);
	}*/
/*	public boolean afterEqualsDate(CscCalendar calendar) {
		return this.equalsDate(calendar) || this.calendar.after(calendar);
	}
*/	
/***************************************/	
	protected boolean equals(Timestamp timestamp) {
		return this.getTimestamp().equals(timestamp);
	}
	public boolean equals(CscCalendar calendar) {
		return this.equals(calendar.getTimestamp());
	}
	protected boolean before(Timestamp timestamp) {
		return this.getTimestamp().before(timestamp);
	}
	public boolean before(CscCalendar calendar) {
		return this.before(calendar.getTimestamp());
	}
	protected boolean after(Timestamp timestamp) {
		return this.getTimestamp().after(timestamp);
	}
	public boolean after(CscCalendar calendar) {
		return this.after(calendar.getTimestamp());
	}
	protected boolean beforeEquals(Timestamp timestamp) {
		return this.equals(timestamp) || this.before(timestamp);
	}
	public boolean beforeEquals(CscCalendar calendar) {
		return this.equals(calendar.getTimestamp()) || this.before(calendar.getTimestamp());
	}
	protected boolean afterEquals(Timestamp timestamp) {
		return this.equals(timestamp) || this.after(timestamp);
	}
	public boolean afterEquals(CscCalendar calendar) {
		return this.equals(calendar) || this.after(calendar);
	}
	//-------------------------------------------------------------------------------------------------------------------------------------
	//  VALUE OF
	//------------------------------------------------------------------------------------------------------------------------------------
	public CscCalendar valueOf(Timestamp timestamp) {
		return new CscCalendar(timestamp);
	}
	public CscCalendar valueOf(java.sql.Date date) {
		return new CscCalendar(date);
	}
	public CscCalendar valueOf(String s) throws CscException {
		return new CscCalendar(s);
	}
	public CscCalendar valueOf(int year, int month, int date) {
		return new CscCalendar(year, month, date);
	}
	public CscCalendar valueOf(int year, int month, int date, int hour, int minute) {
		return new CscCalendar(year, month, date, hour, minute);
	}
	public CscCalendar valueOf(int year, int month, int date, int hour, int minute, int second) {
		return new CscCalendar(year, month, date, hour, minute, second);
	}
	//-----------------------------------------------------------------------------------------------------------------------------------------
	// GET
	//---------------------------------------------------------------------------------------------------------------------------------------
	public int getEra() {
		return myEra;
	}
	/**
	 *  get future of  date by number of day
	 * */
	public CscCalendar getFutureDate(int day) {
		CscCalendar tmp = new CscCalendar(this.calendar.getTime());
		tmp.incDate(day);
		return tmp;
	}
	/**
	 *  Increment date by number of day
	 * */
	public CscCalendar getFutureYear(int year) {
		CscCalendar tmp = new CscCalendar(this.calendar.getTime());
		tmp.incYear(year);
		return tmp;
	}
	public java.util.Date getSqlDate() {
		return this.calendar.getTime();
	}
	public Timestamp getTimestamp() {
		return new Timestamp(this.getTime());
	}
	public long getTime() {
		return this.calendar.getTime().getTime();
	}
        
	public String getString() {
		return new Timestamp(this.getTime()).toString();
	}
	//Balloon modified on 22-1-2004
	public String getString(String format) {
		String date = "";
		if (format.equalsIgnoreCase("d/m/y")) {
			date = this.getDDMMYYHHMMSS();
		} else if (format.equalsIgnoreCase("m/d/y")) {
			date = this.getMMDDYYYYHHMMSS();
		} else if (format.equalsIgnoreCase("y/m/d")) {
			date = this.getYYYYMMDDHHMMSS();
		} else {
			date = this.getString();
		}
		return date;
	}
	/*public float getCSCTime() {
	 float f = (float) this.calendar.get(Calendar.HOUR_OF_DAY) + (this.calendar.get(Calendar.MINUTE) / 100);
	 return f;
	 }*/
	//pae edit
	public float getCSCTime() {
		float f = (float) this.calendar.get(Calendar.HOUR_OF_DAY) + (this.calendar.get(Calendar.MINUTE) / 100f);
		return f;
	}
	/**
	 * Method getCSCMinute.
	 * @return int
	 */
	public String getDefaultFormat() {
		return this.getDDMMYYYY();
	}
	public int getCSCMinute() {
		int m;
		m = (this.calendar.get(Calendar.HOUR_OF_DAY) * 60) + this.calendar.get(Calendar.MINUTE);
		return m;
	}
	public float getDecimal() {
		float f;
		f = this.calendar.get(Calendar.HOUR_OF_DAY) + (this.calendar.get(Calendar.MINUTE) / 60);
		return f;
	}
	public int getDate() {
		return this.calendar.get(Calendar.DATE);
	}
	public int getMonth() {
		return this.calendar.get(Calendar.MONTH);
	}
	public int getCurrentMonth() {
		return this.calendar.get(Calendar.MONTH)+1;
	}
	public int getYear() {
		return this.calendar.get(Calendar.YEAR);
	}
	public int getHour() {
		return this.calendar.get(Calendar.HOUR_OF_DAY);
	}
	public int getMinute() {
		return this.calendar.get(Calendar.MINUTE);
	}
	public int getSecond() {
		return this.calendar.get(Calendar.SECOND);
	}
	public int getDayOfYear() {
		return this.calendar.get(Calendar.DAY_OF_YEAR);
	}
	public int getDayOfWeek() {
		return this.calendar.get(Calendar.DAY_OF_WEEK);
	}
	public int getDayOfMonth() {
		return this.calendar.get(Calendar.DAY_OF_MONTH);
	}
	public int getDayOfWeekInMonth() {
		return this.calendar.get(Calendar.DAY_OF_WEEK_IN_MONTH);
	}
	public String getDD() {
		DecimalFormat form = new DecimalFormat("00");
		return form.format(this.calendar.get(Calendar.DATE));
	}
	public String getDateShort() {
		return this.getDD() + this.getMM() + this.getEngYY();
	}
	public String getTimeShort() {
		DecimalFormat form = new DecimalFormat("00");
		return form.format(this.getHour()) + form.format(this.getMinute()) + form.format(this.getSecond());
	}
	public String getMM() {
		DecimalFormat form = new DecimalFormat("00");
		return form.format(this.calendar.get(Calendar.MONTH) + 1);
	}
	public String getEngYY() {
		return String.valueOf(this.calendar.get(Calendar.YEAR)).substring(2, 4);
	}
	public String getEngYYYY() {
		return String.valueOf(this.calendar.get(Calendar.YEAR)).substring(0, 4);
	}
	public String getThaiYY() {
		return String.valueOf((this.calendar.get(Calendar.YEAR) + 543)).substring(2, 4);
	}
	public String getThaiYYYY() {
		return String.valueOf((this.calendar.get(Calendar.YEAR) + 543)).substring(0, 4);
	}
	public String getDDMMYY() {
		return this.getDD() + this.sign + this.getMM() + this.sign + this.getEngYY();
	}
	public String getDDMMYYYY() {
		return this.getDD() + this.sign + this.getMM() + this.sign + this.getEngYYYY();
	}
	public String getMMDDYY() {
		return this.getMM() + this.sign + this.getDD() + this.sign + this.getEngYY();
	}
	public String getThaiMMDDYY() {
		return this.getMM() + this.sign + this.getDD() + this.sign + this.getThaiYYYY();
	}
	public String getMMDDYYYY() {
		return this.getMM() + this.sign + this.getDD() + this.sign + this.getEngYYYY();
	}
	public String getThaiDDMMYY() {
		return this.getDD() + this.sign + this.getMM() + this.sign + this.getThaiYY();
	}
	public String getThaiMMDDYYYY() {
		return this.getMM() + this.sign + this.getDD() + this.sign + this.getThaiYYYY();
	}
	/**	 
	 * author by lieang 18/06/2005
	 *  input 2005-02-15
	 *  return 15-02-2548
	 * @return
	 */
	public String getThaiDDMMYYYY() {
		return this.getDD() + this.sign + this.getMM() + this.sign + this.getThaiYYYY();
	}
	public String getThaiFullDDMMYYYY(){
		return this.getDD()+" "+this.getThaiLongMonth()+" "+this.getThaiYYYY();
	}	
	public String getYYMMDD() {
		return this.getEngYY() + this.sign + this.getMM() + this.sign + this.getDD();
	}
	public String getYYYYMMDD() {
		return this.getEngYYYY() + this.sign + this.getMM() + this.sign + this.getDD();
	}
	public String getDDMMYYHHMMSS() {
		return this.getDDMMYY() + " " + this.getHHMMSS();
	}
	public String getDDMMYYYYHHMMSS() {
		return this.getDDMMYYYY() + " " + this.getHHMMSS();
	}
	public String getMMDDYYHHMMSS() {
		return this.getMMDDYY() + " " + this.getHHMMSS();
	}
	public String getMMDDYYYYHHMMSS() {
		return this.getDDMMYYYY() + " " + this.getHHMMSS();
	}
	public String getYYMMDDHHMMSS() {
		return this.getYYMMDD() + " " + this.getHHMMSS();
	}
	public String getYYYYMMDDHHMMSS() {
		return this.getYYYYMMDD() + " " + this.getHHMMSS();
	}
	public int getintHH() {
		return this.calendar.get(Calendar.HOUR_OF_DAY);
	}
	public int getintMM() {
		return this.calendar.get(Calendar.MINUTE);
	}
	public int getintSS() {
		return this.calendar.get(Calendar.SECOND);
	}
	public String getHHMMSSFULL() {
		return DateFormat.getTimeInstance(DateFormat.FULL).format(this.calendar.getTime());
	}
	public String getHHMMSS() {
		DecimalFormat form = new DecimalFormat("00");
		String data = "";
		data += form.format((double) this.getHour()) + ":";
		data += form.format((double) this.getMinute()) + ":";
		data += form.format((double) this.getSecond());
		return data;
	}
	public String getDDThaiYY() {
		return this.getDD() + " " + this.getThaiLongMonth() + " " + this.getThaiYY();
	}
	public String getDDEngYY() {
		return this.getDD() + " " + this.getEngLongMonth() + " " + this.getEngYY();
	} //"7 August 00"
	public String getDDEngYYYY() {
		return this.getDD() + " " + this.getEngLongMonth() + " " + this.getEngYYYY();
	} // "7 August 2000"
	public String getDDThaiYYYY() {
		return this.getDD() + " " + this.getThaiLongMonth() + " " + this.getThaiYYYY();
	}
	public String getDDthEngYYYY() {
		return this.getDD() + this.getenDDay(this.calendar.get(Calendar.DAY_OF_MONTH)) + " " + this.getEngLongMonth() + " " + this.getEngYYYY();
	} //  "15th SEPTEMBER 2000"
	public String getEngDDYYYY() {
		return this.getEngLongMonth() + " " + this.getDD() + " " + this.getEngYYYY();
	} // "SEPTEMBER 7, 2000"
	public String getDDthaiYY() {
		return this.getDD() + " " + this.getThaiShortMonth() + " " + this.getThaiYY();
	}
	public String getDDEngSortYY() {
		return this.getDD() + " " + this.getEngShortMonth() + " " + this.getEngYY();
	} //"7 Aug 00";
	public String getDDEngSortYYYY() {
		return this.getDD() + " " + this.getEngShortMonth() + " " + this.getEngYYYY();
	} //"7 Aug 2000";
	public String getDDThaiSortYYYY() {
		return this.getDD() + " " + this.getThaiShortMonth() + " " + this.getThaiYYYY();
	}
	public String getDDThEngYYYY() {
		return this.getDD() + this.getenDDay(this.calendar.get(Calendar.DAY_OF_MONTH)) + " " + this.getEngShortMonth() + " " + this.getEngYYYY();
	} //"15th Sep 2000";
	public String getEngDDSortYYYY() {
		return this.getEngShortMonth() + " " + this.getDD() + " " + this.getEngYYYY();
	} //"Sep 7, 2000";
	public String getEngShortMonth() {
		return this.engShortMonth[this.calendar.get(Calendar.MONTH)];
	} // Jul
	public String getEngLongMonth() {
		return this.engLongMonth[this.calendar.get(Calendar.MONTH)];
	} //"July"
	public String getThaiShortMonth() {
		return this.thaiShortMonth[this.calendar.get(Calendar.MONTH)];
	}
	public String getThaiLongMonth() {
		return this.thaiLongMonth[this.calendar.get(Calendar.MONTH)];
	}
	public String getFirstEngDOW() {
		return "M";
	}
	/**
	 *  Get Short name of day
	 *  Ex. Son Mon
	 * */
	public String getShortEngDOW() {
		int day = this.calendar.get(Calendar.DAY_OF_WEEK) - 1;
		return this.engShortDay[day];
	}
	/**
	 *  Get Long name of day in english
	 *  Ex. Monday Tueday
	 **/
	public String getLongEngDOW() {
		int day = this.calendar.get(Calendar.DAY_OF_WEEK) - 1;
		return this.engLongDay[day];
	}
	/**
	 *  Get Short name of day in thai
	 **/
	public String getShortThiDOW() {
		int day = this.calendar.get(Calendar.DAY_OF_WEEK) - 1;
		return this.thaiShortDay[day];
	}
	/**
	 *  Get Long name of day in thai
	 *  Ex. ѹ ѧ
	 **/
	public String getLongThiDOW() {
		int day = this.calendar.get(Calendar.DAY_OF_WEEK) - 1;
		return this.thaiLongDay[day];
	}
	public int getDOW() {
		return this.calendar.get(Calendar.DAY_OF_WEEK);
	}
	public int getCSCDOW() {
		if (this.calendar.get(Calendar.DAY_OF_WEEK) == 1) {
			return 7;
		} else {
			return this.calendar.get(Calendar.DAY_OF_WEEK) - 1;
		}
	}
	public String getAge(CscCalendar calendar) {
		DecimalFormat form = new DecimalFormat("00");
		DecimalFormat formYear = new DecimalFormat("0000");
		int[] result = this.getArrayAge(calendar);
		return formYear.format(result[0]) + ":" + form.format(result[1]) + ":" + form.format(result[2]);
	}
	/**
	 *  get count of year month and day by divide by 365 for year
	 **/
	public int[] getCountDMY(int day) {
		int[] result = { 0, 0, 0 };
		result[0] = day / 365;
		day -= 365 * result[0];
		result[1] = day / 31;
		day -= 31 * result[1];
		result[2] = day;
		return result;
	}
	/**
	 *  Count day and month in same year
	 *  @return int array[month,day]
	 */
	public int[] getCountDM(CscCalendar endDay, int dayCount) {
		int[] result = { 0, 0 };
		int tmpM = 0, mCount = 0, i = 1;
		while (dayCount > 0) {
			mCount = endDay.getEndDateOfMonth(i);
			if (dayCount >= mCount) {
				tmpM++;
				dayCount -= mCount;
			} else {
				break;
			}
			i++;
		}
		result[0] = tmpM;
		result[1] = dayCount;
		return result;
	}
	public int getDayInMonth() {
		int month = this.calendar.get(Calendar.MONTH);
		if (month == 1) {
			if (this.isLeapYear()) { // is leap year
				return 29;
			} else {
				return 28;
			}
		} else {
			return this.monthCount[month];
		}
	}
	public int getDayInMonth(String date) {
		return new CscCalendar(date).getDayInMonth();
	}
	/**
	 * ʹ㹡õǨͺҧҧ CscCalendar 
	 * 觤ҡѺ Array 纤 [year , month , day]
	 * @param calendar
	 * @return int array[ year , month , day]
	 * */
//	public int[] getCountDMY(CscCalendar calendar) {
//		int[] result = { 0, 0, 0 };
//		int dayCount = this.countDay(calendar);
//		int startY = this.getYear(), endY = calendar.getYear();
//		int tmpY = 0;
//		if (startY == endY) {
//			int[] resultDM = this.getCountDM(calendar, dayCount);
//			result[0] = 0;
//			result[1] = resultDM[0];
//			result[2] = resultDM[1];
//			if (resultDM[0] == 12) { // if month = 12 then increment year
//				result[0] += 1;
//				result[1] = 0;
//			}			
//			return result;
//		} else {
//			for (int i = startY; i < endY; i++) {
//				if ((dayCount > 0) && (dayCount > 364)) {
//					if (this.isLeapYear(i)) {
//						dayCount -= 366;
//						tmpY++;
//					} else {
//						dayCount -= 365;
//						tmpY++;
//					}
//				} else {
//					continue;
//				}
//			}
//			result[0] = tmpY;
//		}
//		if (dayCount > 0) {
//			int[] resultDM = this.getCountDM(calendar, dayCount);
//			result[2] = resultDM[1];
//			if (resultDM[0] == 12) { // if month = 12 then increment year
//				result[0] += 1;
//				result[1] = 0;
//			} else {
//				result[1] = resultDM[0];
//			}
//		}
//		return result;
//	}
	
	/**
	 * write by: james
	 * write date: 2010-11-12
	 */
	public int[] getCountDMY(CscCalendar calendar) {

		int ageYears, ageMonths, ageDays;
		
		CscCalendar startDate = this;
		CscCalendar endDate = calendar;
		if(this.after(calendar)) {
			startDate = calendar;
			endDate = this;
		}
		
		ageYears = endDate.getYear() - startDate.getYear();		
		if (startDate.getMonth() > endDate.getMonth()) {
			ageYears--;
			ageMonths = 12 - startDate.getMonth() + endDate.getMonth();
		}else if (startDate.getMonth() == endDate.getMonth()&&ageYears>0&& startDate.getDate() > endDate.getDate()) { 
			/** óշըӹǹѧú´٨ҡ ѹ͹ ͧѡź͡ **/
			ageYears--;
			ageMonths =0;
		} else {
			ageMonths = endDate.getMonth() - startDate.getMonth();
		}
		if (startDate.getDate() > endDate.getDate()) {			
			ageMonths--;
			/**
			 * edit by pas ͧҡóշӹǳ͡ʹ ageMonths=0 ѹҡѹش ǹź ageMonths ҵԴź
			 * 
			 */
			if(ageMonths<0){
				ageMonths=12-Math.abs(ageMonths);
			}
			/**
			 * end pas 
			 */
			
			int compareMonth;
			int monthT = endDate.getMonth() - 1;
			/**
			 * edit by pas ͧҡóշӹǳ͡ʹ monthT=0 ѹҡѹش ǹź monthT ҵԴź
			 * ͹ҨӹǹѹʺѺѹ
			 * 
			 */
			if(monthT<0){
				monthT=12-Math.abs(monthT);
			}	
			if (monthT == 0 || monthT == 2 || monthT == 4 || monthT == 6 
					|| monthT == 7 || monthT == 9 || monthT == 11) {
				compareMonth = 31;
			} else if (monthT == 3 || monthT == 5 || monthT == 8 || monthT == 10) {
				compareMonth = 30;
			} else if (new GregorianCalendar().isLeapYear(endDate.getYear())) {
				compareMonth = 29;
			} else {
				compareMonth = 28;
			}
			/**
			 * end pas 
			 */				
			ageDays = compareMonth - startDate.getDate() + endDate.getDate();
		} else {
			ageDays = endDate.getDate() - startDate.getDate();
		}
		
		//System.out.println(String.format("YMD=[%d, %d, %d]", ageYears,ageMonths, ageDays ));
		return new int[]{ageYears,ageMonths, ageDays};
	}
	
	public int[] getArrayAge(CscCalendar calendar) {
		/*
		 * int[] result = { 0, 0, 0 };
		Timestamp t0 = this.getTimestamp();
		Timestamp t1 = calendar.getTimestamp();
		long sub = 0;
		if (t0.getTime() > t1.getTime())
			sub = t0.getTime() - t1.getTime();
		else
			sub = t1.getTime() - t0.getTime();
		Long l = new Long(sub / 86400000);
		result = this.getCountDMY(l.intValue() + 1);
		return result;
		*/
		
		int result[]=this.getCountDMY(calendar);
		return result;
	}
	private int getSumDay(String SStart) {
		int s;
		if (d30.indexOf(SStart.substring(4, 6)) != -1) {
			s = 30 - Integer.parseInt(SStart.substring(6)) + 1;
		} else if (d31.indexOf(SStart.substring(4, 6)) != -1) {
			s = 31 - Integer.parseInt(SStart.substring(6)) + 1;
		} else {
			if ((((Integer.parseInt(SStart.substring(4, 6)) % 4) % 100) % 400) == 0) {
				s = 29 - Integer.parseInt(SStart.substring(6)) + 1;
			} else {
				s = 28 - Integer.parseInt(SStart.substring(6)) + 1;
			}
		}
		return s;
	}
	//------ ADD NEW FORMAT BY SOM--------------------------------------------------------------------------------------------------------
	/*Method For get DateFormat
	 * String format : format Off  date  eg. FULL or LONG or MEDIUM or SHORT
	 * String lang : language for output  eg. TH or US
	 * Example :  getTimeFormat("FULL","TH");
	 */
	public String getDateFormat(String format, String lang) {
		Field formatField = null;
		int intformat = 0;
		try {
			Class cd = DateFormat.class;
			Class cl = Locale.class;
			formatField = cd.getDeclaredField(format);
			intformat = formatField.getInt(null);
			Field localeField = cl.getDeclaredField(lang);
			return DateFormat.getDateInstance(intformat, (Locale) localeField.get(null)).format((Date) this.calendar.getTime());
		} catch (Exception e) {
			MyLog.error(this, e);
			return DateFormat.getDateInstance(intformat).format((Date) this.calendar.getTime());
		}
	}
	public int getEndDateOfMonth() {
		CscCalendar cn = new CscCalendar(this.getYear() + this.sign + this.getMM() + "-01 00:00:00");
		cn.incMonth(1);
		cn.decDate(1);
		return cn.getDayOfMonth();
	}
	public int getEndDateOfMonth(int month) {
		String mm = String.valueOf(month);
		if (month<10) {
			mm = "0"+mm;
		}
		CscCalendar cn = new CscCalendar(this.getYear() + this.sign + mm + "-01 00:00:00");
		cn.incMonth(1);
		cn.decDate(1);
		return cn.getDayOfMonth();
	}
	/*
	 * Method For get TimeFormat
	 * String format : format Off  time  eg. FULL or LONG or MEDIUM or SHORT
	 * String lang : language for output  eg. TH or US
	 * Example :  getTimeFormat("FULL","TH");
	 */
	public String getTimeFormat(String format, String lang) {
		Field formatField = null;
		int intformat = 0;
		try {
			Class cd = DateFormat.class;
			Class cl = Locale.class;
			formatField = cd.getDeclaredField(format);
			intformat = formatField.getInt(null);
			Field localeField = cl.getDeclaredField(lang);
			return DateFormat.getTimeInstance(intformat, (Locale) localeField.get(null)).format((Date) this.calendar.getTime());
		} catch (Exception e) {
			MyLog.error(this, e);
			return DateFormat.getTimeInstance(intformat).format((Date) this.calendar.getTime());
		}
	}
	/*
	 * Method For get DateTimeFormat
	 * String dFormat : format Off  date  eg. FULL or LONG or MEDIUM or SHORT
	 * String tFormat : format Off  time eg. FULL or LONG or MEDIUM or SHORT
	 * String lang : language for output  eg. TH or US
	 * Example :  getTimeFormat("FULL","FULL","TH");
	 */
	public String getDateTimeFormat(String dFormat, String tFormat, String lang) {
		Field dFormatField = null, tFormatField = null;
		int intDFormat = 0, intTFormat = 0;
		try {
			Class cd = DateFormat.class;
			Class cl = Locale.class;
			dFormatField = cd.getDeclaredField(dFormat);
			tFormatField = cd.getDeclaredField(tFormat);
			intDFormat = dFormatField.getInt(null);
			intTFormat = tFormatField.getInt(null);
			Field localeField = cl.getDeclaredField(lang);
			return DateFormat.getDateTimeInstance(intDFormat, intTFormat, (Locale) localeField.get(null)).format((Date) this.calendar.getTime());
		} catch (Exception e) {
			MyLog.error(this, e);
			return DateFormat.getDateTimeInstance(intDFormat, intTFormat).format((Date) this.calendar.getTime());
		}
	}
	//-----------------------------------------------------------------------------------------------------------------------------
	//SET METHOD
	//-----------------------------------------------------------------------------------------------------------------------------
	public void setDate(int value) {
		this.calendar.set(Calendar.DATE, value);
	}
	public void setMonth(int value) {
		this.calendar.set(Calendar.MONTH, value);
	}
	public void setYear(int value) {
		this.calendar.set(Calendar.YEAR, value);
	}
	public void setHour(int value) {
		this.calendar.set(Calendar.HOUR_OF_DAY, value);
	}
	public void setMinute(int value) {
		this.calendar.set(Calendar.MINUTE, value);
	}
	public void setSecond(int value) {
		this.calendar.set(Calendar.SECOND, value);
	}
	public void setSign(String sign) {
		this.sign = sign;
	}
	/**
	 * Era have to be 0 = Christian Era, 1 = BuDDhist
	 * */
	public void setEra(int era) {
		if (era <= 1 && era >= 0) {
			this.myEra = era;
			this.calendar.set(Calendar.ERA, era);
		}
	}
	/*public void setCSCTime(float time) throws CscException {
	 int h, m, t;
	 if (time > 0 && time < 24) {
	 time = (time * 100);
	 t = (int) time;
	 String s = String.valueOf(t);
	 if (t < 100) {
	 h = 0;
	 m = new Integer(s).intValue();
	 } else {
	 if (t < 1000) {
	 h = new Integer(s.substring(0, 1)).intValue();
	 m = new Integer(s.substring(1)).intValue();
	 } else {
	 h = new Integer(s.substring(0, 2)).intValue();
	 m = new Integer(s.substring(2)).intValue();
	 }
	 }
	 this.calendar.set(Calendar.HOUR_OF_DAY, h);
	 this.calendar.set(Calendar.MINUTE, m);
	 } else {
	 throw new CscException("Time has something wrong! " + String.valueOf(time));
	 }
	 }*/
	
	public void setCSCTime(double time) throws CscException {
		float t = Float.parseFloat(String.valueOf(time));
		this.setCSCTime(t);
	}
	//pae edit
	public void setCSCTime(float time) throws CscException {
		int h, m, t, status = 0;
		if (time > 0 && time < 24) {
			time = (time * 100);
			t = (int) time;
			String s = String.valueOf(t);
			if (String.valueOf(time).indexOf(".") > -1) {
				String value = String.valueOf(time).substring(String.valueOf(time).indexOf("."), String.valueOf(time).length());
				float f = Float.valueOf(value).floatValue();
				if (f > 0.5)
					status = 1;
				else
					status = 0;
			}
			if (t < 100) {
				h = 0;
				m = new Integer(s).intValue();
			} else {
				if (t < 1000) {
					h = new Integer(s.substring(0, 1)).intValue();
					m = new Integer(s.substring(1)).intValue();
				} else {
					h = new Integer(s.substring(0, 2)).intValue();
					m = new Integer(s.substring(2)).intValue();
				}
			}
			if (status == 1)
				m++;
			//			System.out.println(time);
			//			System.out.println(h);
			//			System.out.println(m);
			this.calendar.set(Calendar.HOUR_OF_DAY, h);
			this.calendar.set(Calendar.MINUTE, m);
			//			System.out.println(this.getHour());
			//			System.out.println(this.getMinute());
			//			this.setHour(h);
			//			this.setMinute(m);
		} else {
			if (time <= 0) {
				this.calendar.set(Calendar.HOUR_OF_DAY, 0);
				this.calendar.set(Calendar.MINUTE, 0);
			} else {
				throw new CscException("Time has something wrong! " + String.valueOf(time));
			}
		}
	}
	public void setTimeInMillis(long arg0) {
		this.calendar.setTimeInMillis(arg0);
	}
	public long getTimeInMillis() {
		return this.calendar.getTimeInMillis();
	}
	public void setTimestamp(String s) {
		if (s.length() <= 10) {
			s += " 00:00:00.00";
		}
		if (s.length() <= 12) {
			s = "1900-01-01 00:00:00";
		}
		this.calendar.setTime((Date) Timestamp.valueOf(s));
		this.dateFormat.setCalendar(this.calendar);
	}
	public void setDecimal(float time) throws CscException {
		int h, m, t;
		if (time > 0 && time < 24) {
			time = (time * 100);
			t = (int) time;
			String s = String.valueOf(t);
			if (t < 100) {
				h = 0;
				m = new Integer(s).intValue();
			} else {
				if (t < 1000) {
					h = new Integer(s.substring(0, 1)).intValue();
					m = new Integer(s.substring(1)).intValue();
				} else {
					h = new Integer(s.substring(0, 2)).intValue();
					m = new Integer(s.substring(2)).intValue();
				}
			}
			if (m > 0) {
				m = (m * 6) / 10;
			}
			this.calendar.set(Calendar.HOUR_OF_DAY, h);
			this.calendar.set(Calendar.MINUTE, m);
		} else {
			throw new CscException("Time has something wrong! " + String.valueOf(time));
		}
	}
	//---------------------------------------------------------------------------------------------------------------------------------------
	// PRIVATE METHOD
	//---------------------------------------------------------------------------------------------------------------------------------------
	/*
	 private void initTimeStamp() {
	 this.timestamp = new Timestamp(this.getTime());
	 }
	 */
	private String getenDDay(int day) {
		String end = "th";
		switch (day) {
			case 1:
				end = "st";
				break;
			case 21:
				end = "st";
				break;
			case 31:
				end = "st";
				break;
			case 2:
				end = "nd";
				break;
			case 22:
				end = "nd";
				break;
			case 3:
				end = "rd";
				break;
			case 23:
				end = "rd";
				break;
		}
		return end;
	}
	/**
	 *  Decrement date by number of date between day1 and day2
	 * */
	public boolean subDayRange(String day1, String day2) {
		try {
			CscCalendar d1 = new CscCalendar(day1);
			int result = d1.subForDay(new CscCalendar(day2));
			if (result > 0)
				this.decDate(result);
			else
				this.decDate(result * -1);
			return true;
		} catch (Exception e) {
			MyLog.error(this, e);

			return false;
		}
	}
	public String toString() {
		return this.getDDMMYYHHMMSS();
	}
	public float DTOT(float tm) {
		// Convert Decimal unit  to Time unit  (12.50 -> 12.30)
		int buff;
		float buff1;
		boolean minus = false;
		if (tm != 0) {
			if (tm < 0) {
				tm = -(tm);
				minus = true;
			}
			buff = (int) tm;
			buff1 = (float) (tm - buff);
			if (buff1 != 0) {
				buff1 = (buff1 * 6) / 10;
				tm = buff + buff1;
			}
			DecimalFormat df = new DecimalFormat("00.00");
			tm = Float.valueOf(df.format((double) tm)).floatValue();
			if (minus) {
				tm = -(tm);
			}
		}
		return tm;
	}
	public float TTOD(float tm) {
		// Convert Time unit to Decimal unit   (12.30 -> 12.50)
		int buff;
		float buff1;
		boolean minus = false;
		if (tm != 0) {
			if (tm < 0) {
				tm = -(tm);
				minus = true;
			}
			buff = (int) tm;
			buff1 = (float) (tm - buff);
			if (buff1 != 0) {
				buff1 = (buff1 * 100) / 60;
				tm = buff + buff1;
			}
			if (minus) {
				tm = -(tm);
			}
		}
		return tm;
	}
	public String getCustomFormat(String format) {
		String dateString = "";
		String symbol = "", temp = "";
		char ch = ' ', tch = ' ';
		int position, next = 0;
		String result = "", tempstr = "";
		String test = "";
		char substr, nextsub;
		char prev = 'f', before = 'f';
		int w = 0, r = 0, st = 0, t;
		boolean outstr = false;
		format = format.toUpperCase();
		dateString = getDateString();
		System.out.println("dateString = " + dateString);
		if (!(format.equalsIgnoreCase(""))) {
			symbol = findSymbol(format, tch);
			System.out.println("symbol = " + symbol);
			//  ó Symbol
			//*********************************************
			while (st < format.length()) {
				substr = format.charAt(st);
				if (substr != 'Y') {
					if (substr == prev) { //͹Ѻǡ͹˹
						result += checkresult(substr, symbol, dateString);
					} else { //Ẻ
						if (substr == 'D' || substr == 'M' || substr == 'Y') {
							//ҵǵ͹ѹ ͹ͧ
							t = st;
							if (++t < format.length()) {
								nextsub = format.charAt(t);
								if (substr != nextsub) {
									result += checkresult(substr, symbol, dateString);
								}
							} else
								result += checkresult(substr, symbol, dateString);
						}
					}
					st++;
				} else { // ó Y
					r = st;
					do {
						w++;
						r++;
						if (r == format.length()) {
							outstr = true;
						}
					} while (!(outstr) && format.charAt(r) == 'Y');
					if (w == 2) {
						if (!todogol) {
							result += getvalue("YY", dateString);
							todogol = true;
						} else
							result += symbol + getvalue("YY", dateString);
					} else if (w == 1 || w == 4) {
						if (!todogol) {
							result += getvalue("YYYY", dateString);
							todogol = true;
						} else
							result += symbol + getvalue("YYYY", dateString);
					}
					st = r;
				}
				prev = substr;
			}
			return result;
		}
		return "";
	}
	private String findSymbol(String x, char tch) {
		int j = 0;
		String tempsymbol = "";
		while (j < x.length()) {
			tch = x.charAt(j);
			if (tch == 'D' || tch == 'M' || tch == 'Y') {
				tempsymbol = "";
			} else {
				tempsymbol = String.valueOf(tch);
				break;
			}
			j += 1;
		}
		return tempsymbol;
	}
	private String checkresult(char substr, String symbol, String dateString) {
		if (!todogol) {
			todogol = true;
			switch (substr) {
				case 'D':
					return getvalue("DD", dateString);
				case 'M':
					return getvalue("MM", dateString);
				default:
					return "";
			}
		} else {
			switch (substr) {
				case 'D':
					return symbol + getvalue("DD", dateString);
				case 'M':
					return symbol + getvalue("MM", dateString);
				default:
					return "";
			}
		}
	}
	private String getvalue(String str, String dateString) {
		if (str.equalsIgnoreCase("DD")) {
			return getD(dateString);
		}
		if (str.equalsIgnoreCase("MM")) {
			return getM(dateString);
		}
		if (str.equalsIgnoreCase("YY")) {
			return getYY(dateString);
		}
		if (str.equalsIgnoreCase("YYYY")) {
			return getYYYY(dateString);
		} else
			return "";
	}
	private String getD(String dateString) {
		return dateString.substring(0, 2);
	}
	private String getM(String dateString) {
		return dateString.substring(3, 5);
	}
	private String getYY(String dateString) {
		return dateString.substring(8, 10);
	}
	private String getYYYY(String dateString) {
		return dateString.substring(6, 10);
	}
	private String getDateString() {
		return getDDMMYYYY();
	}
	public String getDOWeng() {
		return this.engShortDay[this.calendar.get(Calendar.DAY_OF_WEEK) - 1];
	}
	public String getDOWeng0() {
		return "M";
	} // M, T, W, Tu, F, Sa, S
	public String getDOWENG() {
		return this.engLongDay[this.calendar.get(Calendar.DAY_OF_WEEK) - 1];
	} //"Monday"
	public String getDOWthi() {
		return this.thaiShortDay[this.calendar.get(Calendar.DAY_OF_WEEK) - 1];
	}
	public String getDOWTHI() {
		return this.thaiLongDay[this.calendar.get(Calendar.DAY_OF_WEEK) - 1];
	}
	public CscCalendar halfhalp(CscCalendar d) {
		CscCalendar e, b;
		CscCalendar buff = null;
		float gap;
		if (this.before(d)) {
			e = d;
			b = this;
		} else {
			if (this.equals(d)) {
				return buff = new CscCalendar(this.getTimeInMillis());
			}
			e = this;
			b = d;
		}
		gap = e.subForDecimal(b);
		gap = gap / 2;
		gap = e.DTOT(gap);
		buff = new CscCalendar(b.getTimeInMillis());
		buff.incTime(gap);
		return buff;
	}
	/* --------------------------------------------------------------------
	 Count hour for Training 15/09/04
	 Wut edit
	 Input 2 timestamp parameter(2003-01-01 07:00:40.0) and
	 return string HH:MM:SS
	 ---------------------------------------------------------------------*/
	public long countTimeAsLong(String startTime, String endTime) {
		java.sql.Timestamp t1 = new CscCalendar(startTime).getTimestamp();
		java.sql.Timestamp t2 = new CscCalendar(endTime).getTimestamp();
		return t2.getTime() - t1.getTime();
	}
	public String longToDay(long longNumber) {
		return String.valueOf(longNumber / dayOfTimeStamp);
	}
	public String longToHH(long longNumber) {
		return String.valueOf(longNumber / this.hourOfTimeStamp);
	}
	public String longToMM(long longNumber) {
		return String.valueOf(longNumber / this.minuteOfTimeStamp);
	}
	public String longToHHMM(long longNumber) {
		return (longNumber / hourOfTimeStamp) + ":" + ((longNumber / minuteOfTimeStamp) - (longNumber / hourOfTimeStamp) * 60);
	}
	public String longToHHMMSS(long longNumber) {
		return (longNumber / hourOfTimeStamp) + ":" + ((longNumber / minuteOfTimeStamp) - (longNumber / hourOfTimeStamp) * 60) + ":" + ((longNumber / secondOfTimeStamp) - (longNumber / minuteOfTimeStamp) * 60);
	}
	public String getThaiMonth() {
		return this.thaiLongMonth[this.getMonth()];
	}
	public String getHHMM() {
		DecimalFormat form = new DecimalFormat("00");
		String data = "";
		data += form.format((double) this.getHour()) + ":";
		data += form.format((double) this.getMinute());
		return data;
	}
	
	public String getThaiFull(){
		return this.getLongThiDOW()+" "+this.getDate()+" "+this.getThaiLongMonth()+" "+this.getThaiYYYY();
	}
	
	public String getEngFull(){
		return this.getLongEngDOW()+" "+this.getDate()+" "+this.getEngLongMonth()+" "+this.getEngYYYY();
	}
    
    public int getWorkDayInM(){
        int dayInM=this.getEndDateOfMonth();
        for(int i=1;i<=this.getEndDateOfMonth();i++){
            this.setDate(i);
            if(this.getDOWeng().trim().equalsIgnoreCase("sun")||this.getDOWeng().trim().equalsIgnoreCase("sat")){
                dayInM--;
            }else{
               // System.out.println("DOW  Day-->> "+i+"  -->>"+ this.getDOWeng());
            }
        }
        //System.out.println("day in month -->> "+dayInM);
        return dayInM;
    }
    public int getStopDayInM(){
        int dayInM=this.getEndDateOfMonth();
        int Stop=0;
        for(int i=1;i<=this.getEndDateOfMonth();i++){
            this.setDate(i);
            if(this.getDOWeng().trim().equalsIgnoreCase("sun")||this.getDOWeng().trim().equalsIgnoreCase("sat")){
                //dayInM--;
            }else{
                Stop++;
               // System.out.println("DOW  Day-->> "+i+"  -->>"+ this.getDOWeng());
            }
        }
        //System.out.println("day in month -->> "+Stop);
        return Stop;
    }
    
    public int chkStopDate(){
        int typeD=0;
        if(this.getDOWeng().trim().equalsIgnoreCase("sun")||this.getDOWeng().trim().equalsIgnoreCase("sat")){
            typeD=1;
        }else{
            typeD=0;
        }
        return typeD;
    }
    
    public void setDate(Date date) {
    	this.calendar.setTime(date);
    	this.dateFormat.setCalendar(this.calendar);
    }
    
    /**
     * set new date
     * @param date in pattern yyyy-mm-dd 
     */
    public void setDate(String date) {
//    	String yy = date.substring(0,4);
//    	String mm = date.substring(5,7);
//    	String dd = date.substring(8,10);
    	date += " 00:00:00";
    	this.calendar.setTime((Date) Timestamp.valueOf(date));
    	this.dateFormat.setCalendar(this.calendar);
    }
    
    public void  setCalendar(String s){
    	String timestamp="";
    	int charInWord=s.replaceAll("[^a-zA-Z]","").length(); /* ѺǨͺ input ҡյѡ ¡ -  :*/
    	if(charInWord>0){
    		s = "1900-01-01 00:00:00";
    		this.calendar.setTime((Date) Timestamp.valueOf(s));
    	}else{
				s=s.replaceAll("/","-");
				if(s.indexOf("-")>-1 || s.indexOf("/")>-1){
					String dtmp = "", dx = "", mx = "", yx = "";
					int len;
					len = s.length();
					StringTokenizer token = new StringTokenizer(s, "-");
					dx = token.nextToken();
					mx = token.nextToken();
					yx = token.nextToken();
					FileString fs = new FileString();
					if (dx.length()>2) {
						String buff = dx;
						dx = yx;
						yx = buff;
					}
					/** check timestamp **/
					if(dx.length()>2 && dx.indexOf(":")>-1){
						timestamp=" "+dx.substring(3).trim();
						dx=dx.substring(0,2);
					}
					if(yx.length()>4 && yx.indexOf(":")>-1){
						timestamp=" "+yx.substring(5).trim();
						yx=yx.substring(0,4);
					}					
					/**end check timestamp **/					
					if (dx.length() < 2) {
						dx = fs.addBlank(2, "0", "right", dx);
					}
					if (mx.length() < 2) {
						mx = fs.addBlank(2, "0", "right", mx);
					}	
						
						/**
						 * ӡ convert 繻 ..
						 */
						if(Integer.parseInt(yx)>=2443){
							yx=String.valueOf(Integer.parseInt(yx)-543);
						}
						dtmp = yx + "-" + mx + "-" + dx+timestamp;
						s = dtmp;
						if (s.length() <= 10) {
							s += " 00:00:00.00";
						}
						if (s.length() <= 12) {
							s = "1900-01-01 00:00:00";
						}
					this.calendar.setTime((Date) Timestamp.valueOf(s));
		    	}
    	}
		this.dateFormat.setCalendar(this.calendar);
    }
    
   public CscCalendar setCscCalendar(String s){
   				this.setCalendar(s);
    			return this;
    }
   
    public String getDThaiYYYY() {
        return this.getDate() + " " + this.getThaiLongMonth() + " " + this.getThaiYYYY();
    }
   
    public String getDEngYYYY() {
        return this.getDate() + " " + this.getEngLongMonth() + " " + this.getEngYYYY();
    }
    
    private long getDay(){
        GregorianCalendar   cale  = new GregorianCalendar();
        cale.set(Calendar.YEAR,this.getYear());
        cale.set(Calendar.MONTH,this.getMonth());
        cale.set(Calendar.DATE,this.getDate());
        cale.set(Calendar.AM_PM,0);
        cale.set(Calendar.HOUR, 0);
        cale.set(Calendar.MINUTE, 0);
        cale.set(Calendar.SECOND, 0);
        cale.set(Calendar.MILLISECOND, 0);
        return cale.getTime().getTime();
    }
    
    private long getDay(CscCalendar cs){
        GregorianCalendar   cale  = new GregorianCalendar();
        cale.set(Calendar.YEAR,cs.getYear());
        cale.set(Calendar.MONTH,cs.getMonth());
        cale.set(Calendar.DATE,cs.getDate());
        cale.set(Calendar.AM_PM,0);
        cale.set(Calendar.HOUR, 0);
        cale.set(Calendar.MINUTE, 0);
        cale.set(Calendar.SECOND, 0);
        cale.set(Calendar.MILLISECOND, 0);
        return cale.getTime().getTime();
    }
    
    /**
     * 	Letter 	Date or Time Component 	Presentation 	Examples 
     * 	G  	Era designator 	Text  	AD
		y 	Year 	Year 	1996; 96
		M 	Month in year 	Month 	July; Jul; 07
		w 	Week in year 	Number 	27
		W 	Week in month 	Number 	2
		D 	Day in year 	Number 	189
		d 	Day in month 	Number 	10
		F 	Day of week in month 	Number 	2
		E 	Day in week 	Text 	Tuesday; Tue
		a 	Am/pm marker 	Text 	PM
		H 	Hour in day (0-23) 	Number 	0
		k 	Hour in day (1-24) 	Number 	24
		K 	Hour in am/pm (0-11) 	Number 	0
		h 	Hour in am/pm (1-12) 	Number 	12
		m 	Minute in hour 	Number 	30
		s 	Second in minute 	Number 	55
		S 	Millisecond 	Number 	978
		z 	Time zone 	General time zone 	Pacific Standard Time; PST; GMT-08:00
		Z 	Time zone 	RFC 822 time zone 	-0800
     * example
     *     format = dd-MM-yyyy
     *     lang	  = THA 
     *     result = 02-01-2552
     *     
     * @param format
     * @param lang (THA,ENG)
     * @return dateformat
     */
    public String getSimpleDateFormat(String format,String lang){
    	SimpleDateFormat df=null;
    	if(lang.equalsIgnoreCase("ENG")){
    		df=new SimpleDateFormat(format, Locale.ENGLISH);
    	}else{
    		df=new SimpleDateFormat(format, Locale.getDefault());
    	}    	
    	return df.format((Date)this.calendar.getTime());
    }
    
    	public int getWeekOfYear(){
    		return this.calendar.get(Calendar.WEEK_OF_YEAR);
    	}
    
    	public void setWeekOfYear(int weekOfyear){
    		this.calendar.set(Calendar.WEEK_OF_YEAR,weekOfyear);
    	}    	
    
    public static void main(String[] args) {
    	String initdate="2014-01-09  19:30:00";
    	CscCalendar startdate=new CscCalendar(initdate);
    	System.out.println(" "+startdate.getYYYYMMDDHHMMSS());
    	/*CscCalendar startdate=new CscCalendar(initdate);
    	CscCalendar enddate=new CscCalendar(initdate);
    	int year=startdate.getYear();
    	int week=1;
    	while(week<=52){
    		startdate.decDate(startdate.getDayOfWeek()-1);
    		System.out.print("week "+week+" "+startdate.getYYYYMMDD());
    		startdate.incDate(7);
    		enddate=new CscCalendar(startdate.getYYYYMMDD());
    		enddate.decDate();
    		System.out.println(" "+enddate.getYYYYMMDD());
    		week++;
    	}
    	CscCalendar s2=new CscCalendar("2013-12-28");
    	System.out.println(" "+s2.getWeekOfYear());
    	for(int i=1;i<=52;i++){
    		System.out.println("<STARTDATE"+i+" lang=\"A\" encrypt=\"false\" verify=\"false\" log=\"false\" type=\"Date\">\n"+
    				"	<WIDTH>10</WIDTH>\n"+
    				"	<DESC_ENG>Start date</DESC_ENG>\n"+
    				"	<DESC_THAI>ѹ</DESC_THAI>\n"+
    				"	<COMPONENT type=\"text\">\n"+
    				"		<MAXSIZE>10</MAXSIZE>\n"+
    				"		<SIZE>10</SIZE>\n"+
    				"		<ALT_THAI>ѹ</ALT_THAI>\n"+
    				"		<ALT_ENG>Start date </ALT_ENG>\n"+
    				"		<ONKEYPRESS>chkIntegerOfDate(this.value);</ONKEYPRESS>\n"+
    				"		<ONKEYUP>formatDate(this);</ONKEYUP>\n"+
    				"		<ONBLUR>chkInputDate2(this,__enddate);</ONBLUR>\n"+
    				"	</COMPONENT>\n"+
    				"</STARTDATE"+i+">\n"+
    				"<ENDDATE"+i+" lang=\"A\" encrypt=\"false\" verify=\"false\" log=\"false\" type=\"Date\">\n"+
    				"	<WIDTH>10</WIDTH>\n"+
    				"	<DESC_ENG>End date </DESC_ENG>\n"+
    				"	<DESC_THAI>ѹش</DESC_THAI>\n"+
    				"	<COMPONENT type=\"text\">\n"+
    				"		<MAXSIZE>10</MAXSIZE>\n"+
    				"		<SIZE>10</SIZE>\n"+
    				"		<ALT_THAI>ѹش</ALT_THAI>\n"+
    				"		<ALT_ENG>End date </ALT_ENG>\n"+
    				"		<ONKEYPRESS>chkIntegerOfDate(this.value);</ONKEYPRESS>\n"+
    				"		<ONKEYUP>formatDate(this);</ONKEYUP>\n"+
    				"		<ONBLUR>chkInputDate2(__startdate,this);</ONBLUR>\n"+
    				"	</COMPONENT>\n"+
    				"</ENDDATE"+i+">\n"+
    				"<MAXOT"+i+" lang=\"A\" encrypt=\"false\" verify=\"false\" log=\"false\" type=\"Double\">\n"+
    				"	<WIDTH>15</WIDTH>\n"+
    				"	<DESC_ENG>Max OT</DESC_ENG>\n"+
    				"	<DESC_THAI>ӹǹͷ٧ش</DESC_THAI>\n"+
    				"	<COMPONENT type=\"text\">\n"+
    				"		<MAXSIZE>15</MAXSIZE>\n"+
    				"		<SIZE>10</SIZE>\n"+
    				"		<ALT_THAI>ӹǹͷ٧ش</ALT_THAI>\n"+
    				"		<ALT_ENG>Max OT</ALT_ENG>\n"+
    				"		<ONBLUR>checkNumFloat(this,0.00,9999999);</ONBLUR>\n"+
    				"		<ONKEYPRESS>checkWordNumber(this.value);</ONKEYPRESS>\n"+
    				"		<ONKEYUP>checkComma(this,2);</ONKEYUP>\n"+
    				"		<ONFOCUS>callDelComma(this);</ONFOCUS>\n"+
    				"		<DEFAULTVALUE>12.00</DEFAULTVALUE>\n"+
    				"	</COMPONENT>\n"+	
    				"</MAXOT"+i+">");    	
    	}
    	*/
    	/*
    	for(int i=1;i<=52;i++){
    		System.out.println("	<FIELD>STARTDATE"+i+"</FIELD>");
    	 	System.out.println("	<FIELD>ENDDATE"+i+"</FIELD>");
    		System.out.println("	<FIELD>MAXOT"+i+"</FIELD>");
    	}
    	*/
//    	try {
//			s2.setCSCTime(12.50f);
//		} catch (CscException e) {
//			e.printStackTrace();
//		}
//    	s.countDay(s2);
//    	System.out.println(s2);
//    	int dw=s.getDayOfWeek();
//    	System.out.println("date="+s.getDayOfWeek());
//    	if(dw<6){
//    	  s.incDate(6-dw);
//    	}else if(dw>6){
//    		s.incDate(6);
//    	}
//    	System.out.println("date="+s.getDayOfWeek()+" "+s.getDDMMYYYY()+" = "+s.getDayInMonth() );
    	//s.beforeEquals()
//    	System.out.println(s.getSimpleDateFormat("MMMM dd  yyyy","ENG"));
//    	System.out.println(s.getDayInMonth());
//    	System.out.println("date+"+s.getDate()+"=>"+s.getYear()+"-"+s.getMM()+"-"+s.getDD());
//    	s.incDate(30);
//    	CscCalendar start = new CscCalendar("2012-09-20");
//    	start.incDate(30);
//    	CscCalendar end = new CscCalendar(start.getYYYYMMDD());
//    	start.decDate(30);
//    	int age[]=start.getCountDMY(end);
//    	double as=start.countDay(end);
//    	
//    	System.out.println("==="+end.getYYYYMMDD());
//    	System.out.println(String.format("YMD = [%d, %d, %d]", age[0],age[1], age[2]));
//    	System.out.println("1312527743765l"+new CscCalendar(1312527743765l));
//    	start.incYear(1);
//    	System.out.println("calendar="+start.getYYYYMMDD()+" ");
//    	start.decMonth(10);
//    	System.out.println("calendar"+start.getYYYYMMDD()+" ");
//    	int maxday=30;
//    	String month="0";
//    	for(int m=1;m<=12;m++){
//    		if(m==2){
//    			maxday=28;
//    			}else if(m==1 ||m==3 ||m==5 ||m==7 ||m==8 ||m==10 ||m==12){
//    				maxday=31;
//    			}else{
//    				maxday=30;
//    		}
//    		if(m<10){
//    			month="0"+m;
//    		}else{
//    			month=""+m;
//    		}
//    		for(int i=1;i<=maxday;i++){
//		    	final CscCalendar end = new CscCalendar("2011-"+month+"-"+i);
//		    	int age[] = start.getCountDMY(end);
//				System.out.println(end.getYYYYMMDD()+"="+age[0]+"."+age[1]+"."+age[2]);
//    		}
//    	}
//		
//		System.out.println(String.format("YMD = [%d, %d, %d]", age[0],age[1], age[2] ));
//        CscCalendar csc=new CscCalendar();
//        csc.setMonth(0);
//        csc.setDate(1);
		
	}
}
