package com.csc.library.utilities;

import java.io.File;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.Element;

import com.csc.library.session.InitialEnvironment;

public class FlagProtable {
	
	private Element root;
	private Element wroot;
	private Vector tables;
	

	public FlagProtable() {
		this.initial();
	}
		
	public void initial() {
		MyLog.info(this,"Start Initial Config File ");
		String path = new InitialEnvironment("GLOBAL").getValue("config-dir");
		// get root of flagconfig
		ReadXml rx = new ReadXml(path + "FlagConfig.xml");
		this.root = rx.getRootElement();
		if (this.root == null) 
			MyLog.debug(this, "Can't find File >>> FlagConfig.xml");
		this.tables = new Vector(this.root.getChildren());
		MyLog.info(this, "End Initial Config File");
	}
	
	public void processReplaceProTable() {
		MyLog.info(this, "Start Process Replace XML File");
		String propath = new InitialEnvironment("GLOBAL").getValue("xmlpath-protable");
		File files = new File(propath);
		String[] filenames = files.list();
		this.replaceFlag(filenames,propath);
		MyLog.info(this, "End Process Replace XML File");
	}
	
	private void replaceFlag(String[] filelist,String path) {
		for (int i = 0; i < filelist.length; i++) {
			String filename = filelist[i];
			if (filename.equalsIgnoreCase("cvs") 
					|| filename.equalsIgnoreCase("setting"))
				continue;
			String full = path + filename;
			File file = new File(full + "/");
			if (file.isDirectory()) {
				MyLog.info(this, "Sub Folder Name >>> " + filename);
				this.replaceFlag(file.list(), path + filename + "/");
			} else {
				int dotPoint = filename.indexOf(".");
				if (dotPoint > 0) {
					filename = filename.substring(0, filename.indexOf(".")).toLowerCase();
				} else {
					MyLog.error(this, "Can't sub string file name >>" + filename+ "<< Skip process this file");
					continue;
				}
				for (int j = 0; j < this.tables.size(); j++) {
					String tablename = ((Element) this.tables.get(j)).getAttributeValue("name");
					// check tablename in config file and protable
					if (filename.indexOf(tablename) > -1) {
						ReadXml rx = new ReadXml(full);
						// get root of protable file
						Element rootxml = rx.getRootElement();
						String rootName = rootxml.getName();
						boolean updatexml = false;
						for (int k = 0; k < this.tables.size(); k++) {
							Element tableconfig = (Element) this.tables.get(k);
							String tableName = tableconfig
									.getAttributeValue("name");
							// compare tablename between configfile and protable
							if (rootName.equalsIgnoreCase(tableName)) {
								Vector fields = new Vector(tableconfig.getChildren());
								for (int x = 0; x < fields.size(); x++) {
									Element field = (Element) fields.get(x);
									// get field from protable
									Element proField = rootxml.getChild("FIELD")
											.getChild(field.getAttributeValue("name").toUpperCase());
									if (proField != null) {
										Vector attrs = new Vector(field.getChildren());
										for (int y = 0; y < attrs.size(); y++) {
											Element attr = (Element) attrs.get(y);
											String attrname = attr.getName().toLowerCase();
											Attribute proattr = proField.getAttribute(attrname);
											updatexml = true;
											if (proattr != null) {
												String provalue = proattr.getValue();
												String configvalue = attr.getAttributeValue("value");
												if (!provalue.equalsIgnoreCase(configvalue)) {
													proattr.setValue(configvalue);
													proField.removeAttribute(attrname);
													proField.addAttribute(proattr);
												}
											} else {
												String configvalue = attr.getAttributeValue("value");
												Attribute newattr = new Attribute(attrname,configvalue);
												proField.addAttribute(newattr);
											}
										}
									} else {
										continue;
									}
								}
								break;
							}
						}
						if (updatexml) {
							WriteXml wx = new WriteXml(rootxml, full);
							wx.output();
						}
					}
				}
			}
		}
	}
	
	public void processGenConfig(String attrs) {
		MyLog.info(this, "Start Generate ConfigFile");
		String path = new InitialEnvironment("GLOBAL").getValue("config-dir");
		this.wroot = new Element("DATABASE");
		String propath = new InitialEnvironment("GLOBAL").getValue("xmlpath-protable");
		File files = new File(propath);
		String[] filenames = files.list();
		this.createDataEncrypt(filenames,propath,attrs);
		int attempt = 1;
		String configfilepath = "";
		boolean ok = false;
		while (!ok) {
			configfilepath = path+"FlagConfig"+attempt+".xml";
			File chkFile = new File(configfilepath);
			ok = !chkFile.exists();
			attempt++;
		}
		MyLog.info(this, "Gen File config(Encrypt Attribute Only!!) >>> " + configfilepath);
		WriteXml wx = new WriteXml(this.wroot, configfilepath);
		wx.setNewLines(true);
		wx.output();
		MyLog.info(this, "End Generate ConfigFile");
	}
	
	private void createDataEncrypt(String[] filelist,String path,String attrs) {
		for (int i = 0; i < filelist.length; i++) {
			String filename = filelist[i];
			if (filename.equalsIgnoreCase("cvs") 
					|| filename.equalsIgnoreCase("setting"))
				continue;
			String fullfilepath = path + filename;
			File file = new File(fullfilepath + "/");
			if (file.isDirectory()) {
				MyLog.info(this, "Sub Folder Name >>> " + filename);
				this.createDataEncrypt(file.list(), path + filename + "/",attrs);
			} else {
				ReadXml rx = new ReadXml(fullfilepath);
				Element proRoot = rx.getRootElement();
				Element tableElement = this.genTableElement(attrs,proRoot);
				if (tableElement != null)
					this.wroot.addContent(tableElement);
			}
		}
	}
	
	private Element genTableElement(String attrs,Element source) {
		Element newTable = new Element("TABLE");
		String tablename = source.getName();
		newTable.addAttribute("name",tablename);
		Iterator iter = source.getChild("FIELD").getChildren().iterator();
		boolean haveField = false;
		while (iter.hasNext()) {
			Element field = (Element) iter.next();
			Element configfield = this.genFieldElement(attrs,field);
			if (configfield != null && !this.inRoot(tablename,configfield)) {
				haveField = true;
				newTable.addContent(configfield);
			}
		}
		if (haveField) {
			MyLog.debug(this, " > Gen Table -> " + tablename);
			return newTable;
		}
		else 
			return null;
	}
	
	private Element genFieldElement(String attrs,Element source) {
		Element newfield = new Element("FIELD");
		String fieldname = source.getName();
		newfield.addAttribute("name",fieldname);
		StringTokenizer stn = new StringTokenizer(attrs,",");
		boolean haveAttr = false;
		while (stn.hasMoreTokens()) {
			String attrname = stn.nextToken();
			String attrvalue = source.getAttributeValue(attrname.toLowerCase());
			if (attrvalue != null && attrvalue.equalsIgnoreCase("true")) {
				haveAttr = true;
				Element newChild = new Element(attrname.toUpperCase());
				newChild.addAttribute("value",attrvalue);
				newfield.addContent(newChild);
			}
		}
		if (haveAttr){
			MyLog.debug(this, "  > Gen Field --> " + fieldname);
			return newfield;
		} else
			return null;
	}
	
	private boolean inRoot(String tablename,Element fieldelement) {
		boolean result = false;
		boolean havetable = false;
		Iterator itables = this.wroot.getChildren().iterator();
		Vector allresult = new Vector();
		while (itables.hasNext() && !havetable) {
			Element table = (Element) itables.next();
			if (table.getAttributeValue("name").equalsIgnoreCase(tablename)) {
				havetable = true;
				Iterator ifields = table.getChildren().iterator();
				boolean havefield = false;
				while (ifields.hasNext() && !havefield) {
					Element field = (Element) ifields.next();
					if (field.getAttributeValue("name").equalsIgnoreCase(fieldelement.getAttributeValue("name"))) {
						havefield = true;
						boolean haveattr = false;
						Iterator iattrs = field.getChildren().iterator();
						while (iattrs.hasNext()) {
							Element attr = (Element) iattrs.next();
							if (fieldelement.getChild(attr.getName()) != null) {
								haveattr = true;
							} 
						}
						if (!haveattr) {
							havefield = false;
							break;
						}
					}
				}
				if (!havefield) {
					havetable = false;
					break;
				}
			}
			allresult.add(String.valueOf(havetable));
		}
		for (int i=0; i<allresult.size(); i++) {
			String tmpresult = (String) allresult.get(i);
			if (tmpresult.equalsIgnoreCase("true")) {
				result = true;
			}
		}
		return result;
	}
		
}
